# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This file contains a markdown story exporter."""

from __future__ import unicode_literals

import datetime
import markdown


from timesketch.lib.stories import interface
from timesketch.lib.stories import manager


class HTMLStoryExporter(interface.StoryExporter):
    """HTML story exporter."""

    # String representing the output format of the story.
    EXPORT_FORMAT = 'html'

    # Number of rows of a DataFrame to include at the top of a markdown table.
    _DATAFRAME_HEADER_ROWS = 30

    HTML_HEADER = (
        '<html>'
        '    <head>'
        '<script src="https://cdn.jsdelivr.net/npm/vega@5"></script>'
        '<script src="https://cdn.jsdelivr.net/npm/vega-lite@4"></script>'
        ' <script src="https://cdn.jsdelivr.net/npm/vega-embed@6"></script>'
        ' </head>'
        ' <body>')

    HTML_FOOTER = '</body></html>'

    def _data_frame_to_html(self, data_frame):
        """Returns a HTML formatted string from a pandas DataFrame."""
        nr_rows, _ = data_frame.shape
        if not nr_rows:
            return '<b>empty table</b>'

        if nr_rows <= self._DATAFRAME_HEADER_ROWS:
            return data_frame.to_html(index=False)

        html_code = data_frame.to_html(
            index=False, max_rows=self._DATAFRAME_HEADER_ROWS)
        return (
            '{0:s}\n<b>...</b><br/>'
            '<i>Table contains more records than are displayed. '
            'Only the first {1:d} records out of total {2:d} are '
            'printed out. View the full view in Timesketch to get all '
            'the results.</i>').format(
                html_code, self._DATAFRAME_HEADER_ROWS, nr_rows)

    def export_story(self):
        """Export the story as HTML."""
        return_strings = [self.HTML_HEADER]

        return_strings.append(
            '<h1>{0:s}</h1>'.format(self._story_title))
        date_now = datetime.datetime.utcnow()
        return_strings.append(
            'Story Metadata:<br/><ul><li><b>Creation Date:</b> {0:s}</li>'
            '<li><b>Created by:</b> {1:s}</li>'
            '<li><b>Report Generated:</b> {2:s}</li>'
            '<li><b>Report Exported by:</b> {3:s}</li></ul>'.format(
                self._creation_date, self._story_author, date_now.isoformat(),
                self._story_exporter))

        md = markdown.Markdown()

        chart_number = 1
        charts = []
        for line_dict in self._data_lines:
            line_type = line_dict.get('type', '')
            if line_type == 'text':
                return_strings.append(
                    md.convert(line_dict.get('value', '')))

            elif line_type == 'aggregation':
                data_dict = line_dict.get('value')
                agg_obj = data_dict.get('aggregation')
                chart_type = data_dict.get('chart_type')
                if chart_type == 'table':
                    df = agg_obj.to_pandas()
                    return_strings.append(self._data_frame_to_html(df))
                else:
                    parameters = data_dict.get('parameters', {})
                    title = parameters.get('title', data_dict.get('name'))
                    chart_color = parameters.get('chart_color')

                    chart = agg_obj.to_chart(
                        chart_name=chart_type, chart_title=title,
                        interactive=True, color=chart_color, as_chart=True)

                    charts.append(chart)
                    return_strings.append(
                        '<div id="vis{0:d}"></div>'.format(chart_number))
                    chart_number += 1

            elif line_type == 'dataframe':
                return_strings.append(
                    self._data_frame_to_html(line_dict.get('value')))

            elif line_type == 'chart':
                data_dict = line_dict.get('value')
                chart = data_dict.get('chart')

                charts.append(chart)
                return_strings.append(
                    '<div id="vis{0:d}"></div>'.format(chart_number))
                chart_number += 1

        if charts:
            return_strings.append('<script type="text/javascript">')
            for index, chart in enumerate(charts):
                vis_nr = index + 1
                return_strings.append(
                    'vegaEmbed(\'#vis{0:d}\', {1:s}).'
                    'catch(console.error);'.format(
                        vis_nr, chart.to_json(indent=None)))
            return_strings.append('</script>')

        return_strings.append('</body></html>')
        return '\n\n'.join(return_strings)


manager.StoryExportManager.register_exporter(HTMLStoryExporter)



"""Tests for BrowserSearchPlugin."""
from __future__ import unicode_literals

import mock

from timesketch.lib.analyzers import browser_search
from timesketch.lib.testlib import BaseTest
from timesketch.lib.testlib import MockDataStore


class TestBrowserSearchPlugin(BaseTest):
    """Tests the functionality of the analyzer."""

    # Mock the Elasticsearch datastore.
    @mock.patch(
        'timesketch.lib.analyzers.interface.ElasticsearchDataStore',
        MockDataStore)
    def test_url_extraction(self):
        """Test the browser search analyzer."""
        analyzer = browser_search.BrowserSearchSketchPlugin('test_index', 1)

        # Need to access protected members for testing purposes.
        # pylint: disable=protected-access
        bing_search = 'https://www.bing.com/search?q=foobar+stuff&f=en'
        bing_result = analyzer._extract_search_query_from_url(bing_search, 'q')
        self.assertEqual(bing_result, 'foobar stuff')

        github_search = 'https://github.com/search?q=foobar&another=true'
        github_result = analyzer._extract_search_query_from_url(
            github_search, 'q')
        self.assertEqual(github_result, 'foobar')

        drive_search = (
            'https://drive.google.com/drive/search'
            '?q=my%20secret%20stuff&ln=en')
        drive_result = analyzer._extract_search_query_from_url(
            drive_search, 'q')
        self.assertEqual(drive_result, 'my secret stuff')

        google_search = (
            'https://subsite.google.is/search?q=my%20sec'
            'ret%20stuff&ln=en')
        google_result = analyzer._extract_search_query_from_url(
            google_search, 'q')
        self.assertEqual(google_result, 'my secret stuff')
        google_search = 'https://google.com/search?q=my%20secret%20stuff&ln=en'
        google_result = analyzer._extract_search_query_from_url(
            google_search, 'q')
        self.assertEqual(google_result, 'my secret stuff')

        sites_search = (
            'https://sites.google.com/site/mydomain/system/app/pages/'
            'meta/search?q=my%20secret%20stuff&ln=en')
        sites_result = analyzer._extract_search_query_from_url(
            sites_search, 'q')
        self.assertEqual(sites_result, 'my secret stuff')

        duck_search = 'https://duckduckgo.com/?q=foobar+stuff&f=en'
        duck_result = analyzer._extract_search_query_from_url(
            duck_search, 'q')
        self.assertEqual(duck_result, 'foobar stuff')

        duck_search = 'https://duckduckgo.com/?q=from%3Ame%40stuff.com&f=en'
        duck_result = analyzer._extract_search_query_from_url(
            duck_search, 'q')
        self.assertEqual(duck_result, 'from:me@stuff.com')

        group_search = (
            'https://groups.google.com/a/mydomain/forum/#!search'
            '/secret/somestuff')
        group_result = analyzer._extract_urlpart_search_query(group_search)
        self.assertEqual(group_result, 'secret')



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Line Chart object."""

from __future__ import unicode_literals

import altair as alt

from timesketch.lib.charts import manager
from timesketch.lib.charts import interface


class LineChart(interface.BaseChart):
    """Line Chart object."""

    NAME = 'linechart'

    def generate(self):
        """Generate the chart.

        Returns:
            Instance of altair.Chart
        """
        chart = self._get_chart_with_transform()
        self._add_url_href(self.encoding)

        if self.chart_title:
            chart = chart.mark_line().properties(title=self.chart_title)
        else:
            chart = chart.mark_line()
        chart.encoding = alt.FacetedEncoding.from_dict(self.encoding)
        return chart


manager.ChartManager.register_chart(LineChart)



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Timeline resources for version 1 of the Timesketch API."""

import codecs
import json
import logging
import uuid
import six

from flask import request
from flask import abort
from flask import current_app
from flask_restful import Resource
from flask_login import login_required
from flask_login import current_user

from timesketch.api.v1 import resources
from timesketch.lib import forms
from timesketch.lib.definitions import HTTP_STATUS_CODE_OK
from timesketch.lib.definitions import HTTP_STATUS_CODE_CREATED
from timesketch.lib.definitions import HTTP_STATUS_CODE_BAD_REQUEST
from timesketch.lib.definitions import HTTP_STATUS_CODE_FORBIDDEN
from timesketch.lib.definitions import HTTP_STATUS_CODE_NOT_FOUND
from timesketch.models import db_session
from timesketch.models.sketch import SearchIndex
from timesketch.models.sketch import Sketch
from timesketch.models.sketch import Timeline


logger = logging.getLogger('timesketch.timeline_api')


class TimelineListResource(resources.ResourceMixin, Resource):
    """Resource to get all timelines for sketch."""

    @login_required
    def get(self, sketch_id):
        """Handles GET request to the resource.

        Returns:
            View in JSON (instance of flask.wrappers.Response)
        """
        sketch = Sketch.query.get_with_acl(sketch_id)
        if not sketch:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND, 'No sketch found with this ID.')
        if not sketch.has_permission(current_user, 'read'):
            abort(HTTP_STATUS_CODE_FORBIDDEN,
                  'User does not have read access controls on sketch.')
        return self.to_json(sketch.timelines)

    @login_required
    def post(self, sketch_id):
        """Handles POST request to the resource.

        Returns:
            A sketch in JSON (instance of flask.wrappers.Response)
        """
        sketch = Sketch.query.get_with_acl(sketch_id)
        if not sketch:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND, 'No sketch found with this ID.')

        if not sketch.has_permission(current_user, 'write'):
            abort(HTTP_STATUS_CODE_FORBIDDEN,
                  'User does not have write access controls on sketch.')
        form = forms.AddTimelineSimpleForm.build(request)
        metadata = {'created': True}

        searchindex_id = form.timeline.data
        searchindex = SearchIndex.query.get_with_acl(searchindex_id)
        if searchindex.get_status.status == 'deleted':
            abort(
                HTTP_STATUS_CODE_BAD_REQUEST,
                'Unable to create a timeline using a deleted search index')

        timeline_id = [
            t.searchindex.id for t in sketch.timelines
            if t.searchindex.id == searchindex_id
        ]

        if not form.validate_on_submit():
            abort(
                HTTP_STATUS_CODE_BAD_REQUEST, 'Unable to validate form data.')

        if not sketch.has_permission(current_user, 'write'):
            abort(
                HTTP_STATUS_CODE_FORBIDDEN,
                'User does not have write access to the sketch.')

        if not timeline_id:
            return_code = HTTP_STATUS_CODE_CREATED
            timeline = Timeline(
                name=searchindex.name,
                description=searchindex.description,
                sketch=sketch,
                user=current_user,
                searchindex=searchindex)
            sketch.timelines.append(timeline)
            labels_to_prevent_deletion = current_app.config.get(
                'LABELS_TO_PREVENT_DELETION', [])

            for label in sketch.get_labels:
                if label not in labels_to_prevent_deletion:
                    continue
                timeline.add_label(label)
                searchindex.add_label(label)

            # Set status to ready so the timeline can be queried.
            timeline.set_status('ready')

            db_session.add(timeline)
            db_session.commit()
        else:
            metadata['created'] = False
            return_code = HTTP_STATUS_CODE_OK
            timeline = Timeline.query.get(timeline_id)

        # Run sketch analyzers when timeline is added. Import here to avoid
        # circular imports.
        # pylint: disable=import-outside-toplevel
        if current_app.config.get('AUTO_SKETCH_ANALYZERS'):
            # pylint: disable=import-outside-toplevel
            from timesketch.lib import tasks
            sketch_analyzer_group, _ = tasks.build_sketch_analysis_pipeline(
                sketch_id, searchindex_id, current_user.id)
            if sketch_analyzer_group:
                pipeline = (tasks.run_sketch_init.s(
                    [searchindex.index_name]) | sketch_analyzer_group)
                pipeline.apply_async()

        return self.to_json(
            timeline, meta=metadata, status_code=return_code)


class TimelineResource(resources.ResourceMixin, Resource):
    """Resource to get timeline."""

    def _add_label(self, timeline, label):
        """Add a label to the timeline."""
        if timeline.has_label(label):
            logger.warning(
                'Unable to apply the label [{0:s}] to timeline {1:s}, '
                'already exists.'.format(label, timeline.name))
            return False
        timeline.add_label(label, user=current_user)
        return True

    def _remove_label(self, timeline, label):
        """Removes a label from a timeline."""
        if not timeline.has_label(label):
            logger.warning(
                'Unable to remove the label [{0:s}] from timeline {1:s}, '
                'label does not exist.'.format(label, timeline.name))
            return False
        timeline.remove_label(label)
        return True

    @login_required
    def get(self, sketch_id, timeline_id):
        """Handles GET request to the resource.

        Args:
            sketch_id: Integer primary key for a sketch database model
            timeline_id: Integer primary key for a timeline database model
        """
        sketch = Sketch.query.get_with_acl(sketch_id)
        if not sketch:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND, 'No sketch found with this ID.')
        timeline = Timeline.query.get(timeline_id)

        if not timeline:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND, 'No Timeline found with this ID.')

        # Check that this timeline belongs to the sketch
        if timeline.sketch_id != sketch.id:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND,
                'The sketch ID ({0:d}) does not match with the timeline '
                'sketch ID ({1:d})'.format(sketch.id, timeline.sketch_id))

        if not sketch.has_permission(user=current_user, permission='read'):
            abort(
                HTTP_STATUS_CODE_FORBIDDEN,
                'The user does not have read permission on the sketch.')

        return self.to_json(timeline)

    @login_required
    def post(self, sketch_id, timeline_id):
        """Handles GET request to the resource.

        Args:
            sketch_id: Integer primary key for a sketch database model
            timeline_id: Integer primary key for a timeline database model
        """
        sketch = Sketch.query.get_with_acl(sketch_id)
        if not sketch:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND, 'No sketch found with this ID.')
        timeline = Timeline.query.get(timeline_id)
        if not timeline:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND,
                'No timeline found with this ID.')

        # Check that this timeline belongs to the sketch
        if timeline.sketch_id != sketch.id:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND,
                'The sketch ID ({0:d}) does not match with the timeline '
                'sketch ID ({1:d})'.format(sketch.id, timeline.sketch_id))

        if not sketch.has_permission(user=current_user, permission='write'):
            abort(
                HTTP_STATUS_CODE_FORBIDDEN,
                'The user does not have write permission on the sketch.')

        form = forms.TimelineForm.build(request)
        if not form.validate_on_submit():
            abort(
                HTTP_STATUS_CODE_BAD_REQUEST, 'Unable to validate form data.')

        if form.labels.data:
            label_string = form.labels.data
            labels = json.loads(label_string)
            if not isinstance(labels, (list, tuple)):
                abort(
                    HTTP_STATUS_CODE_BAD_REQUEST, (
                        'Label needs to be a JSON string that '
                        'converts to a list of strings.'))
            if not all([isinstance(x, str) for x in labels]):
                abort(
                    HTTP_STATUS_CODE_BAD_REQUEST, (
                        'Label needs to be a JSON string that '
                        'converts to a list of strings (not all strings)'))

            label_action = form.label_action.data
            if label_action not in ('add', 'remove'):
                abort(
                    HTTP_STATUS_CODE_BAD_REQUEST,
                    'Label action needs to be either add or remove.')

            changed = False
            if label_action == 'add':
                changes = []
                for label in labels:
                    changes.append(
                        self._add_label(timeline=timeline, label=label))
                changed = any(changes)
            elif label_action == 'remove':
                if not sketch.has_permission(
                        user=current_user, permission='delete'):
                    abort(
                        HTTP_STATUS_CODE_FORBIDDEN,
                        'The user does not have delete permission on sketch.')

                changes = []
                for label in labels:
                    changes.append(
                        self._remove_label(timeline=timeline, label=label))
                changed = any(changes)

            if not changed:
                abort(
                    HTTP_STATUS_CODE_BAD_REQUEST,
                    'Label [{0:s}] not {1:s}'.format(
                        ', '.join(labels), label_action))

            db_session.add(timeline)
            db_session.commit()
            return HTTP_STATUS_CODE_OK

        timeline.name = form.name.data
        timeline.description = form.description.data
        timeline.color = form.color.data
        db_session.add(timeline)
        db_session.commit()

        return HTTP_STATUS_CODE_OK

    @login_required
    def delete(self, sketch_id, timeline_id):
        """Handles DELETE request to the resource.

        Args:
            sketch_id: Integer primary key for a sketch database model
            timeline_id: Integer primary key for a timeline database model
        """
        sketch = Sketch.query.get_with_acl(sketch_id)
        if not sketch:
            abort(
                HTTP_STATUS_CODE_NOT_FOUND, 'No sketch found with this ID.')
        timeline = Timeline.query.get(timeline_id)

        # Check that this timeline belongs to the sketch
        if timeline.sketch_id != sketch.id:
            if not timeline:
                msg = 'No timeline found with this ID.'
            elif not sketch:
                msg = 'No sketch found with this ID.'
            else:
                msg = (
                    'The sketch ID ({0:d}) does not match with the timeline '
                    'sketch ID ({1:d})'.format(sketch.id, timeline.sketch_id))
            abort(HTTP_STATUS_CODE_NOT_FOUND, msg)

        if not sketch.has_permission(user=current_user, permission='write'):
            abort(
                HTTP_STATUS_CODE_FORBIDDEN,
                'The user does not have write permission on the sketch.')

        not_delete_labels = current_app.config.get(
            'LABELS_TO_PREVENT_DELETION', [])
        for label in not_delete_labels:
            if timeline.has_label(label):
                abort(
                    HTTP_STATUS_CODE_FORBIDDEN,
                    'Timelines with label [{0:s}] cannot be deleted.'.format(
                        label))

        sketch.timelines.remove(timeline)
        db_session.commit()
        return HTTP_STATUS_CODE_OK


class TimelineCreateResource(resources.ResourceMixin, Resource):
    """Resource to create a timeline."""

    @login_required
    def post(self):
        """Handles POST request to the resource.

        Returns:
            A view in JSON (instance of flask.wrappers.Response)
        """
        upload_enabled = current_app.config['UPLOAD_ENABLED']
        if not upload_enabled:
            abort(
                HTTP_STATUS_CODE_BAD_REQUEST,
                'Failed to create timeline, upload not enabled')

        form = forms.CreateTimelineForm()
        if not form.validate_on_submit():
            abort(
                HTTP_STATUS_CODE_BAD_REQUEST,
                'Failed to create timeline, form data not validated')

        sketch_id = form.sketch_id.data
        timeline_name = form.name.data

        sketch = None
        if sketch_id:
            sketch = Sketch.query.get_with_acl(sketch_id)
            if not sketch:
                abort(
                    HTTP_STATUS_CODE_NOT_FOUND,
                    'No sketch found with this ID.')

        # We do not need a human readable filename or
        # datastore index name, so we use UUIDs here.
        index_name = uuid.uuid4().hex
        if not isinstance(index_name, six.text_type):
            index_name = codecs.decode(index_name, 'utf-8')

        # Create the search index in the Timesketch database
        searchindex = SearchIndex.get_or_create(
            name=timeline_name,
            description=timeline_name,
            user=current_user,
            index_name=index_name)
        searchindex.grant_permission(permission='read', user=current_user)
        searchindex.grant_permission(permission='write', user=current_user)
        searchindex.grant_permission(
            permission='delete', user=current_user)
        searchindex.set_status('processing')
        db_session.add(searchindex)
        db_session.commit()

        timeline = None
        if sketch and sketch.has_permission(current_user, 'write'):
            timeline = Timeline(
                name=searchindex.name,
                description=searchindex.description,
                sketch=sketch,
                user=current_user,
                searchindex=searchindex)
            sketch.timelines.append(timeline)
            db_session.add(timeline)
            db_session.commit()

        # Return Timeline if it was created.
        # pylint: disable=no-else-return
        if timeline:
            return self.to_json(
                timeline, status_code=HTTP_STATUS_CODE_CREATED)

        return self.to_json(
            searchindex, status_code=HTTP_STATUS_CODE_CREATED)



# Copyright 2018 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Emoji codepoint definitions.

See https://emojipedia.org for list of available unicode emojis.
"""

from __future__ import unicode_literals

import collections


emoji = collections.namedtuple('emoji', 'code help')


EMOJI_MAP = {
    'BUCKET': emoji('&#x1FAA3', 'Storage bucket'),
    'CAMERA': emoji('&#x1F4F7', 'Screenshot activity'),
    'FISHING_POLE': emoji('&#x1F3A3', 'Phishing'),
    'GLOBE': emoji('&#x1F30F', 'The world'),
    'ID_BUTTON': emoji('&#x1F194', 'Account ID'),
    'LINK': emoji('&#x1F517', 'Events Linked'),
    'LOCK': emoji('&#x1F512', 'Logon activity'),
    'LOCOMOTIVE': emoji('&#x1F682', 'Execution activity'),
    'MAGNIFYING_GLASS': emoji('&#x1F50E', 'Search related activity'),
    'PERSON_STANDING': emoji('&#x1F9CD', 'Person'),
    'SATELLITE': emoji('&#x1F4E1', 'Domain activity'),
    'SCREEN': emoji('&#x1F5B5', 'Screensaver activity'),
    'SKULL': emoji('&#x1F480;', 'Threat intel match'),
    'SKULL_CROSSBONE': emoji('&#x2620', 'Suspicious entry'),
    'SLEEPING_FACE': emoji('&#x1F634', 'Activity outside of regular hours'),
    'SPARKLES': emoji('&#x2728', 'New entity created'),
    'UNLOCK': emoji('&#x1F513', 'Logoff activity'),
    'WASTEBASKET': emoji('&#x1F5D1', 'Deletion activity')
}


def get_emoji(name):
    """Returns a Unicode for an emoji given the name or blank if not saved.

    Args:
        name: string with the emoji name.

    Returns:
        Unicode string for the emoji if it exists or a blank string otherwise.
    """
    name_upper = name.upper()
    emoji_object = EMOJI_MAP.get(name_upper)
    if emoji_object:
        return emoji_object.code
    return ''


def get_helper_from_unicode(code):
    """Returns a helper string from an emoji Unicode code point.

    Args:
        code: a Unicode code point for an emoji.

    Returns:
        Helper text as a string or an empty string if emoji is not configured.
    """
    code_upper = code.upper()
    for emoji_object in iter(EMOJI_MAP.values()):
        if code_upper == emoji_object.code.upper():
            return emoji_object.help
    return ''


def get_emojis_as_dict():
    """Returns a dictionary with emoji codes and helper texts.

    Returns:
        Dict with emoji unicode code points as key and helper text as value.
    """
    return {e.code: e.help for e in iter(EMOJI_MAP.values())}



# -*- coding: utf-8 -*-
"""Plugin for chaining Chrome downloads to filesystem and execution events."""

from timesketch.lib.analyzers.chain_plugins import interface
from timesketch.lib.analyzers.chain_plugins import manager


class ChromeDownloadFilesystemChainPlugin(interface.BaseChainPlugin):
    """A plugin to chain Chrome downloads to filesystem events."""

    NAME = 'chromefilesystem'
    DESCRIPTION = (
        'Plugin to chain Chrome download records to corresponding filesystem '
        'events and execution events.')

    SEARCH_QUERY = 'data_type:"chrome:history:file_downloaded"'
    EVENT_FIELDS = ['full_path']

    def get_chained_events(self, base_event):
        """Yields an event that is chained or linked to the base event.

        Args:
            base_event: the base event of the chain, used to construct further
                queries (instance of Event).

        Yields:
            An event (instance of Event) object that is linked or chained to
            the base event, according to the plugin.
        """
        target = base_event.source.get('full_path', '')
        if not target:
            return
            yield  # pylint: disable=W0101

        if '\\' in target:
            separator = '\\'
        else:
            separator = '/'

        target = target.split(separator)[-1]
        # TODO: Add more checks here, eg; USB, generic execution, etc.
        search_query = (
            '(data_type:"fs:stat" AND filename:"*{0:s}") OR '
            '(data_type:"fs:stat:ntfs" AND name:"{0:s}")').format(
                target)
        return_fields = ['filename', 'path_hints']

        events = self.analyzer_object.event_stream(
            search_query, return_fields=return_fields, scroll=False)
        for event in events:
            yield event

        exec_query = 'executable:"*{0:s}"'.format(target)
        return_fields = ['executable', 'chains']

        events = self.analyzer_object.event_stream(
            exec_query, return_fields=return_fields, scroll=False)
        for event in events:
            yield event


manager.ChainPluginsManager.register_plugin(ChromeDownloadFilesystemChainPlugin)



"""Tests for BrowserTimeframePlugin."""
from __future__ import unicode_literals

import numpy as np
import pandas as pd

from timesketch.lib.analyzers import browser_timeframe
from timesketch.lib.testlib import BaseTest


class TestBrowserTimeframePlugin(BaseTest):
    """Tests the functionality of the analyzer."""

    def test_get_list_of_consecutive_sequences(self):
        """Test the get_list_of_consecutive_sequences function."""
        hours = [0, 1, 2, 3, 9, 10, 11, 12, 13, 14]
        runs = browser_timeframe.get_list_of_consecutive_sequences(hours)

        self.assertEqual(len(runs), 2)

        first_run = runs[0]
        self.assertEqual(first_run[0], 0)
        self.assertEqual(first_run[1], 3)

        second_run = runs[1]
        self.assertEqual(second_run[0], 9)
        self.assertEqual(second_run[1], 14)

        hours = [0, 2, 3, 9, 10, 12, 14]
        runs = browser_timeframe.get_list_of_consecutive_sequences(hours)
        self.assertEqual(len(runs), 5)

    def test_fix_gap_in_list(self):
        """Test the fix_gap_in_list function."""
        hours = [0, 6, 10, 11, 13, 14]
        fixed_hours = browser_timeframe.fix_gap_in_list(hours)

        self.assertEqual(fixed_hours, [0, 10, 11, 12, 13, 14])

    def test_get_active_hours(self):
        """Test get_active_hours function."""
        date_array = np.random.randn(500)
        hours = [
            np.repeat(0, 80),
            np.repeat(1, 50),
            np.repeat(4, 10),
            np.repeat(7, 8),
            np.repeat(12, 12),
            np.repeat(18, 40),
            np.repeat(19, 50),
            np.repeat(20, 10),
            np.repeat(21, 90),
            np.repeat(22, 50),
            np.repeat(23, 100),
        ]

        data_frame = pd.DataFrame({
            'hour': np.concatenate(hours),
            'datetime': date_array})

        expected_hours = [0, 1, 18, 19, 20, 21, 22, 23]
        active_hours, _, _ = browser_timeframe.get_active_hours(data_frame)
        self.assertEqual(active_hours, expected_hours)



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SearchTemplate resources for version 1 of the Timesketch API."""

from flask import abort
from flask_restful import Resource
from flask_login import login_required

from timesketch.api.v1 import resources
from timesketch.lib.definitions import HTTP_STATUS_CODE_NOT_FOUND
from timesketch.models.sketch import SearchTemplate


class SearchTemplateResource(resources.ResourceMixin, Resource):
    """Resource to get a search template."""

    @login_required
    def get(self, searchtemplate_id):
        """Handles GET request to the resource.

        Args:
            searchtemplate_id: Primary key for a search template database model

        Returns:
            Search template in JSON (instance of flask.wrappers.Response)
        """
        searchtemplate = SearchTemplate.query.get(searchtemplate_id)
        if not searchtemplate:
            abort(HTTP_STATUS_CODE_NOT_FOUND, 'Search template was not found')
        return self.to_json(searchtemplate)


class SearchTemplateListResource(resources.ResourceMixin, Resource):
    """Resource to create a search template."""

    @login_required
    def get(self):
        """Handles GET request to the resource.

        Returns:
            View in JSON (instance of flask.wrappers.Response)
        """
        return self.to_json(SearchTemplate.query.all())



#!/usr/bin/python
# Copyright 2015 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module is for management of the Timesketch application."""
from __future__ import unicode_literals

import codecs
import os
import pwd
import re
import sys
import uuid
import yaml

import six

from flask import current_app
from werkzeug.exceptions import Forbidden

from flask_migrate import MigrateCommand
from flask_script import Command
from flask_script import Manager
from flask_script import Server
from flask_script import Option
from flask_script import prompt_bool
from flask_script import prompt_pass
from sqlalchemy.exc import IntegrityError

from timesketch import version
from timesketch.app import create_app
from timesketch.lib.datastores.elastic import ElasticsearchDataStore
from timesketch.models import db_session
from timesketch.models import drop_all
from timesketch.models.user import Group
from timesketch.models.user import User
from timesketch.models.sketch import SearchIndex
from timesketch.models.sketch import SearchTemplate
from timesketch.models.sketch import Sketch
from timesketch.models.sketch import Timeline


class GetVersion(Command):
    """Returns the version information of Timesketch."""

    # pylint: disable=method-hidden
    def run(self):
        """Return the version information of Timesketch."""
        return 'Timesketch version: {0:s}'.format(
            version.get_version())


class DropDataBaseTables(Command):
    """Drop all database tables."""

    # pylint: disable=method-hidden
    def run(self):
        """Drop all tables after user ha verified."""
        verified = prompt_bool(
            'Do you really want to drop all the database tables?')
        if verified:
            sys.stdout.write('All tables dropped. Database is now empty.\n')
            drop_all()


class AddUser(Command):
    """Create a new Timesketch user."""
    option_list = (
        Option('--username', '-u', dest='username', required=True),
        Option('--password', '-p', dest='password', required=False), )

    def get_password_from_prompt(self):
        """Get password from the command line prompt."""
        first_password = prompt_pass('Enter password')
        second_password = prompt_pass('Enter password again')
        if first_password != second_password:
            sys.stderr.write('Passwords don\'t match, try again.\n')
            self.get_password_from_prompt()
        return first_password

    # pylint: disable=arguments-differ, method-hidden
    def run(self, username, password):
        """Creates the user."""
        if not password:
            password = self.get_password_from_prompt()
        if not isinstance(password, six.text_type):
            password = codecs.decode(password, 'utf-8')
        if not isinstance(username, six.text_type):
            username = codecs.decode(username, 'utf-8')
        user = User.get_or_create(username=username)
        user.set_password(plaintext=password)
        db_session.add(user)
        db_session.commit()
        sys.stdout.write('User {0:s} created/updated\n'.format(username))


class MakeUserAdmin(Command):
    """Make user into an administrator."""
    option_list = (
        Option('--username', '-u', dest='username', required=True),
        Option(
            '--remove', '-r', dest='remove', action='store_true',
            required=False, default=False),
    )

    # pylint: disable=arguments-differ, method-hidden
    def run(self, username, remove):
        """Adds the admin bit to a user."""
        user = User.query.filter_by(username=username).first()

        if not user:
            sys.stdout.write('User [{0:s}] does not exist.\n'.format(
                username))
            return
        user.admin = not remove
        db_session.add(user)
        db_session.commit()

        if remove:
            sys.stdout.write('User {0:s} is no longer an admin.\n'.format(
                username))
        else:
            sys.stdout.write('User {0:s} is now an admin.\n'.format(username))


class ListUsers(Command):
    """List all users."""

    # pylint: disable=arguments-differ, method-hidden
    def run(self):
        """The run method for the command."""
        for user in User.query.all():
            if user.admin:
                extra = ' (admin)'
            else:
                extra = ''
            print('{0:s}{1:s}'.format(user.username, extra))


class AddGroup(Command):
    """Create a new Timesketch group."""
    option_list = (Option('--name', '-n', dest='name', required=True), )

    # pylint: disable=arguments-differ, method-hidden
    def run(self, name):
        """Creates the group."""
        if not isinstance(name, six.text_type):
            name = codecs.decode(name, 'utf-8')
        group = Group.get_or_create(name=name)
        db_session.add(group)
        db_session.commit()
        sys.stdout.write('Group {0:s} created\n'.format(name))


class ListGroups(Command):
    """List all groups."""

    # pylint: disable=arguments-differ, method-hidden
    def run(self):
        """The run method for the command."""
        for group in Group.query.all():
            print(group.name)


class GroupManager(Command):
    """Manage group memberships."""
    option_list = (
        Option(
            '--remove',
            '-r',
            dest='remove',
            action='store_true',
            required=False,
            default=False),
        Option(
            '--expand',
            dest='expand',
            action='store_true',
            required=False,
            default=False),
        Option('--group', '-g', dest='group_name', required=True),
        Option('--user', '-u', dest='user_name', required=False, default=None)
    )

    # pylint: disable=arguments-differ, method-hidden
    def run(self, remove, expand, group_name, user_name):
        """Add the user to the group."""
        if not isinstance(group_name, six.text_type):
            group_name = codecs.decode(group_name, 'utf-8')

        group = Group.query.filter_by(name=group_name).first()

        # List all members of a group and then exit.
        if expand:
            for _user in group.users:
                print(_user.username)
            return

        if not isinstance(user_name, six.text_type):
            user_name = codecs.decode(user_name, 'utf-8')
        user = None
        if user_name:
            user = User.query.filter_by(username=user_name).first()

        # Add or remove user from group
        if remove and user:
            try:
                user.groups.remove(group)
                sys.stdout.write('{0:s} removed from group {1:s}\n'.format(
                    user_name, group_name))
                db_session.commit()
            except ValueError:
                sys.stdout.write('{0:s} is not a member of group {1:s}\n'.
                                 format(user_name, group_name))
        elif user:
            user.groups.append(group)
            try:
                db_session.commit()
                sys.stdout.write('{0:s} added to group {1:s}\n'.format(
                    user_name, group_name))
            except IntegrityError:
                sys.stdout.write('{0:s} is already a member of group {1:s}\n'.
                                 format(user_name, group_name))


class AddSearchIndex(Command):
    """Create a new Timesketch searchindex."""
    option_list = (
        Option('--name', '-n', dest='name', required=True),
        Option('--index', '-i', dest='index', required=True),
        Option('--user', '-u', dest='username', required=True), )

    # pylint: disable=arguments-differ, method-hidden
    def run(self, name, index, username):
        """Create the SearchIndex."""
        es = ElasticsearchDataStore(
            host=current_app.config['ELASTIC_HOST'],
            port=current_app.config['ELASTIC_PORT'])
        user = User.query.filter_by(username=username).first()
        if not user:
            sys.stderr.write('User does not exist\n')
            sys.exit(1)
        if not es.client.indices.exists(index=index):
            sys.stderr.write('Index does not exist in the datastore\n')
            sys.exit(1)
        if SearchIndex.query.filter_by(name=name, index_name=index).first():
            sys.stderr.write(
                'Index with this name already exist in Timesketch\n')
            sys.exit(1)
        searchindex = SearchIndex(
            name=name, description=name, user=user, index_name=index)
        db_session.add(searchindex)
        db_session.commit()
        searchindex.grant_permission('read')
        sys.stdout.write('Search index {0:s} created\n'.format(name))


class PurgeTimeline(Command):
    """Delete timeline permanently from Timesketch and Elasticsearch."""
    option_list = (Option(
        '--index', '-i', dest='index_name', required=True), )

    # pylint: disable=arguments-differ, method-hidden
    def run(self, index_name):
        """Delete timeline in both Timesketch and Elasticsearch.

        Args:
            index_name: The name of the index in Elasticsearch
        """
        if not isinstance(index_name, six.text_type):
            index_name = codecs.decode(index_name, 'utf-8')

        searchindex = SearchIndex.query.filter_by(
            index_name=index_name).first()

        if not searchindex:
            sys.stdout.write('No such index\n')
            sys.exit()

        es = ElasticsearchDataStore(
            host=current_app.config['ELASTIC_HOST'],
            port=current_app.config['ELASTIC_PORT'])

        timelines = Timeline.query.filter_by(searchindex=searchindex).all()
        sketches = [
            t.sketch for t in timelines
            if t.sketch and t.sketch.get_status.status != 'deleted'
        ]
        if sketches:
            sys.stdout.write('WARNING: This timeline is in use by:\n')
            for sketch in sketches:
                sys.stdout.write(' * {0:s}\n'.format(sketch.name))
                sys.stdout.flush()
        really_delete = prompt_bool(
            'Are you sure you want to delete this timeline?')
        if really_delete:
            for timeline in timelines:
                db_session.delete(timeline)
            db_session.delete(searchindex)
            db_session.commit()
            es.client.indices.delete(index=index_name)


class SearchTemplateManager(Command):
    """Command Module to manipulate Search templates."""
    option_list = (
        Option('--import', '-i', dest='import_location', required=False),
        Option('--export', '-e', dest='export_location', required=False),
    )

    # pylint: disable=arguments-differ, method-hidden
    def run(self, import_location, export_location):
        """Export/Import search templates to/from file.

        Args:
            import_location: Path to the yaml file to import templates.
            export_location: Path to the yaml file to export templates.
        """

        if export_location:
            search_templates = []
            for search_template in SearchTemplate.query.all():
                labels = []
                for label in search_template.labels:
                    if label.label.startswith('supported_os:'):
                        labels.append(label.label.replace(
                            'supported_os:', ''))
                search_templates.append({
                    'name': search_template.name,
                    'query_string': search_template.query_string,
                    'query_dsl': search_template.query_dsl,
                    'supported_os': labels
                })

            with open(export_location, 'w') as fh:
                yaml.safe_dump(search_templates, stream=fh)

        if import_location:
            try:
                with open(import_location, 'rb') as fh:
                    search_templates = yaml.safe_load(fh)
            except IOError as e:
                sys.stdout.write('Unable to open file: {0!s}\n'.format(e))
                sys.exit(1)

            for search_template in search_templates:
                name = search_template['name']
                query_string = search_template['query_string']
                query_dsl = search_template['query_dsl']

                # Skip search template if already exits.
                if SearchTemplate.query.filter_by(name=name).first():
                    continue

                imported_template = SearchTemplate(
                    name=name,
                    user=User(None),
                    query_string=query_string,
                    query_dsl=query_dsl)

                # Add supported_os labels.
                for supported_os in search_template['supported_os']:
                    label_name = 'supported_os:{0:s}'.format(supported_os)
                    label = SearchTemplate.Label.get_or_create(
                        label=label_name, user=None)
                    imported_template.labels.append(label)

                # Set flag to identify local vs import templates.
                remote_flag = SearchTemplate.Label.get_or_create(
                    label='remote_template', user=None)
                imported_template.labels.append(remote_flag)

                db_session.add(imported_template)
                db_session.commit()


class ListSketches(Command):
    """List all available sketches."""

    # pylint: disable=arguments-differ, method-hidden
    def run(self):
        """The run method for the command."""
        sketches = Sketch.query.all()

        name_len = max([len(x.name) for x in sketches])
        desc_len = max([len(x.description) for x in sketches])

        if not name_len:
            name_len = 5
        if not desc_len:
            desc_len = 10

        fmt_string = '{{0:^3d}} | {{1:{0:d}s}} | {{2:{1:d}s}}'.format(
            name_len, desc_len)

        print('+-'*40)
        print(' ID | Name {0:s} | Description'.format(' '*(name_len-5)))
        print('+-'*40)
        for sketch in sketches:
            status = sketch.get_status.status
            if status == 'deleted':
                continue

            if status == 'archived':
                name = '{0:s} (archived)'.format(sketch.name)
            else:
                name = sketch.name

            print(fmt_string.format(
                sketch.id, name, sketch.description))
            print('-'*80)


class ImportTimeline(Command):
    """Create a new Timesketch timeline from a file."""
    option_list = (
        Option('--file', '-f', dest='file_path', required=True),
        Option('--sketch_id', '-s', dest='sketch_id', required=False),
        Option('--username', '-u', dest='username', required=False),
        Option('--timeline_name', '-n', dest='timeline_name',
               required=False),
    )

    FILENAME_RE = re.compile(r'(^\d+)_.+\.(plaso|csv|jsonl)')

    # pylint: disable=arguments-differ, method-hidden
    def run(self, file_path, sketch_id, username, timeline_name):
        """This is the run method."""

        file_path = os.path.realpath(file_path)
        file_path_no_extension, extension = os.path.splitext(file_path)
        extension = extension.lstrip('.')
        filename = os.path.basename(file_path_no_extension)

        supported_extensions = ('plaso', 'csv', 'jsonl')

        if not os.path.isfile(file_path):
            sys.exit('No such file: {0:s}'.format(file_path))

        if extension not in supported_extensions:
            sys.exit(
                'Extension {0:s} is not supported. '
                '(supported extensions are: {1:s})'.format(
                    extension, ', '.join(supported_extensions)))

        user = None
        if not username:
            username = pwd.getpwuid(os.stat(file_path).st_uid).pw_name
        if not username == 'root':
            if not isinstance(username, six.text_type):
                username = codecs.decode(username, 'utf-8')
            user = User.query.filter_by(username=username).first()
        if not user:
            sys.exit('Cannot determine user for file: {0:s}'.format(file_path))

        sketch = None
        # If filename starts with <number> then use that as sketch_id.
        # E.g: 42_file_name.plaso means sketch_id is 42.
        file_match = self.FILENAME_RE.match(filename)
        if file_match:
            sketch_id_from_filename = file_match.groups()[0]
        else:
            sketch_id_from_filename = ''

        if not sketch_id and sketch_id_from_filename.isdigit():
            sketch_id = sketch_id_from_filename

        if sketch_id:
            if not sketch_id.isdigit():
                sys.exit('Sketch ID needs to be a number, not a string.')
            try:
                sketch = Sketch.query.get_with_acl(sketch_id, user=user)
            except Forbidden:
                pass

        if not timeline_name:
            if timeline_name is None:
                timeline_name = '{0:s}_timeline'.format(filename)

            if not isinstance(timeline_name, six.text_type):
                timeline_name = codecs.decode(timeline_name, 'utf-8')

            timeline_name = timeline_name.replace('_', ' ')
            # Remove sketch ID if present in the filename.
            timeline_parts = timeline_name.split()
            if timeline_parts[0].isdigit():
                timeline_name = ' '.join(timeline_name.split()[1:])

        if not sketch:
            # Create a new sketch.
            sketch_name = 'Sketch for: {0:s}'.format(timeline_name)
            sketch = Sketch(
                name=sketch_name, description=sketch_name, user=user)
            # Need to commit here to be able to set permissions later.
            db_session.add(sketch)
            db_session.commit()
            sketch.grant_permission(permission='read', user=user)
            sketch.grant_permission(permission='write', user=user)
            sketch.grant_permission(permission='delete', user=user)
            sketch.status.append(sketch.Status(user=None, status='new'))
            db_session.add(sketch)
            db_session.commit()

        index_name = uuid.uuid4().hex
        if not isinstance(index_name, six.text_type):
            index_name = codecs.decode(index_name, 'utf-8')

        searchindex = SearchIndex.get_or_create(
            name=timeline_name,
            description=timeline_name,
            user=user,
            index_name=index_name)

        searchindex.grant_permission(permission='read', user=user)
        searchindex.grant_permission(permission='write', user=user)
        searchindex.grant_permission(permission='delete', user=user)

        searchindex.set_status('processing')
        db_session.add(searchindex)
        db_session.commit()

        if sketch and sketch.has_permission(user, 'write'):
            timeline = Timeline(
                name=searchindex.name,
                description=searchindex.description,
                sketch=sketch,
                user=user,
                searchindex=searchindex)
            timeline.set_status('processing')
            sketch.timelines.append(timeline)
            db_session.add(timeline)
            db_session.commit()

        # Start Celery pipeline for indexing and analysis.
        # Import here to avoid circular imports.
        # pylint: disable=import-outside-toplevel
        from timesketch.lib import tasks
        pipeline = tasks.build_index_pipeline(
            file_path=file_path, events='', timeline_name=timeline_name,
            index_name=index_name, file_extension=extension,
            sketch_id=sketch.id)
        pipeline.apply_async(task_id=index_name)

        print('Imported {0:s} to sketch: {1:d} ({2:s})'.format(
            file_path, sketch.id, sketch.name))


def main():
    """Main function of the script, setting up the shell manager."""
    # Setup Flask-script command manager and register commands.
    shell_manager = Manager(create_app)
    shell_manager.add_command('add_user', AddUser())
    shell_manager.add_command('make_admin', MakeUserAdmin())
    shell_manager.add_command('list_users', ListUsers())
    shell_manager.add_command('add_group', AddGroup())
    shell_manager.add_command('list_groups', ListGroups)
    shell_manager.add_command('manage_group', GroupManager())
    shell_manager.add_command('add_index', AddSearchIndex())
    shell_manager.add_command('db', MigrateCommand)
    shell_manager.add_command('drop_db', DropDataBaseTables())
    shell_manager.add_command('list_sketches', ListSketches())
    shell_manager.add_command('purge', PurgeTimeline())
    shell_manager.add_command('search_template', SearchTemplateManager())
    shell_manager.add_command('import', ImportTimeline())
    shell_manager.add_command('version', GetVersion())
    shell_manager.add_command('runserver',
                              Server(host='127.0.0.1', port=5000))
    shell_manager.add_option(
        '-c',
        '--config',
        dest='config',
        default='/etc/timesketch/timesketch.conf',
        required=False)
    shell_manager.run()


if __name__ == '__main__':
    main()



# Copyright 2018 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for Cloud IAP."""

from __future__ import unicode_literals

import time
import mock
import jwt

from cryptography.hazmat.backends.openssl.rsa import _RSAPublicKey

from timesketch.lib.testlib import BaseTest
from timesketch.lib.google_auth import decode_jwt
from timesketch.lib.google_auth import validate_jwt
from timesketch.lib.google_auth import get_public_key_for_jwt
from timesketch.lib.google_auth import JwtValidationError
from timesketch.lib.google_auth import JwtKeyError


# openssl ecparam -genkey -name prime256v1 -noout -out ec_private.pem
MOCK_EC_PRIVATE_KEY = """
-----BEGIN EC PRIVATE KEY-----
MHcCAQEEINoHHwc7+3FSbIg3UMS8w21G58EprN+OT6zGmP93vdhEoAoGCCqGSM49
AwEHoUQDQgAEkLZi2KNrKBnpRKvol8w+ZyaBdjtVdhXMONj3fbMzQNRt5/NvLbq6
1fWR3kDQQXp0Jq2hT0SrZn9M2gT5uqaUCQ==
-----END EC PRIVATE KEY-----
"""

# openssl ec -in ec_private.pem -pubout -out ec_public.pem
MOCK_EC_PUBLIC_KEY = """
-----BEGIN PUBLIC KEY-----
MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEkLZi2KNrKBnpRKvol8w+ZyaBdjtV
dhXMONj3fbMzQNRt5/NvLbq61fWR3kDQQXp0Jq2hT0SrZn9M2gT5uqaUCQ==
-----END PUBLIC KEY-----
"""

MOCK_INVALID_EC_PUBLIC_KEY = """
-----BEGIN PUBLIC KEY-----
MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAE45WYklVqGutB1h/v0FFOtjeACMA0
62e1qPRxoXqi/vREEDnRdkjAq0SlS+pGFVeZq89NUFYwfMBIySF0ShppPg==
-----END PUBLIC KEY-----
"""

# pylint: disable=line-too-long
# Created with https://mkjwk.org/
MOCK_RSA_PRIVATE_PUBLIC_KEY_JWK = {
    "kty": "RSA",
    "d": "DlJPaEHlCOtPg2qIQ3sFYDJRG-JnHVaz4jJoICLP-BDyo6oVBqGuiia2TdZ1OM_1Df1PCk0sfFyacQw0i1Cno-b2ypZZrAgU5zH8tXiBoFSTu6xp2eeLLPeVKueigbR9Uf0NU3BZ2WcxHojq3T6pil1Au7bMzff0JIPXWcECcO4eLOAqPupYswKO8ebT-sl-wT_AtKqCCxAWKbbSpkgHXLNB7k5A-R7m-hiIK5rSQWoqJu-5SROYzCCADCiwRH5tXbaWLWvFFtiF5rl8VnHHRAPFSEG0iIe6-KlsOFdPA2qdiXgJ-Qb39vxOZ5g8qlCRTU7eshvMKs5dtyBtSTo4eQ",
    "e": "AQAB",
    "use": "sig",
    "kid": "oidc_1234",
    "alg": "RS256",
    "n": "jVUkmrTXhFmaahZExVcdJqb3BqZp2A6Kk-IFkmeLimK2DJg3OpUSxEJ5mlaymu7XQJUlG2qKI7zhL7WV-S9CNYdLCVWMhg_XQ9dKB9VoYf92eAufkGrl2GbGd0y6KdMrTuxGfESC-l-exTcQAPvn1Md95difnruob6K1KXQTqEqQEFhKLiciFtssyiC90r8ia7-082MSJUXpXNhHyyuehuV5Xs5GCVqZfP65MMiDiKidUxq40UNTodqJCzhum7iSK42SF9la7ao0FTizF1uFl7oU7fIbN2qzNBgn5U3CTfaZaII54Xn6pzoIwinYXCZzeTy7x-8ZIN41VDNGulTK_w"
}

# pylint: disable=line-too-long
MOCK_RSA_PUBLIC_KEY_JWK = {
    "kty": "RSA",
    "e": "AQAB",
    "use": "sig",
    "kid": "oidc_1234",
    "alg": "RS256",
    "n": "jVUkmrTXhFmaahZExVcdJqb3BqZp2A6Kk-IFkmeLimK2DJg3OpUSxEJ5mlaymu7XQJUlG2qKI7zhL7WV-S9CNYdLCVWMhg_XQ9dKB9VoYf92eAufkGrl2GbGd0y6KdMrTuxGfESC-l-exTcQAPvn1Md95difnruob6K1KXQTqEqQEFhKLiciFtssyiC90r8ia7-082MSJUXpXNhHyyuehuV5Xs5GCVqZfP65MMiDiKidUxq40UNTodqJCzhum7iSK42SF9la7ao0FTizF1uFl7oU7fIbN2qzNBgn5U3CTfaZaII54Xn6pzoIwinYXCZzeTy7x-8ZIN41VDNGulTK_w"
}

# pylint: disable=line-too-long
MOCK_INVALID_RSA_PUBLIC_KEY_JWK = {
    "kty": "RSA",
    "e": "AQAB",
    "use": "sig",
    "kid": "invalid_oidc_1234",
    "alg": "RS256",
    "n": "tQjrbztEuU3GFrNzgxCyb2lAeMXBQPsMctyaQAx05JLiqIxWYE__oSgMXyGz9SR7F_cCZ-x0FVQO1UjIyHU1BUPQxlO88NwEXlIrP5Eo_dnKzn972Cl4AjmoISApp2dTlwhSkHRqycOxJApzsqsg2HQxbrtM6oZrOyFU3uc2u0PHJqVAO6W7fplZXRHRGMd7KaT2a4GJ8zfNHD157Mv7ttjCeQ7rmqkB6OLVpNzO8idKo33gz5gDID2f7IGIPlqpAOAZaaKkRlGqUoaG0EZHns3fPcl27AvNOvmvOl9Blb43UnbBIBFGm6MY9aJUEJzmqcX-td9mch5HtGG14v3sUQ"
}

# Converted to PEM format with pem-jwk (npm install -g pem-jwk)
MOCK_RSA_PRIVATE_KEY_PEM = """
-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEAjVUkmrTXhFmaahZExVcdJqb3BqZp2A6Kk+IFkmeLimK2DJg3
OpUSxEJ5mlaymu7XQJUlG2qKI7zhL7WV+S9CNYdLCVWMhg/XQ9dKB9VoYf92eAuf
kGrl2GbGd0y6KdMrTuxGfESC+l+exTcQAPvn1Md95difnruob6K1KXQTqEqQEFhK
LiciFtssyiC90r8ia7+082MSJUXpXNhHyyuehuV5Xs5GCVqZfP65MMiDiKidUxq4
0UNTodqJCzhum7iSK42SF9la7ao0FTizF1uFl7oU7fIbN2qzNBgn5U3CTfaZaII5
4Xn6pzoIwinYXCZzeTy7x+8ZIN41VDNGulTK/wIDAQABAoIBAA5ST2hB5QjrT4Nq
iEN7BWAyURviZx1Ws+IyaCAiz/gQ8qOqFQahroomtk3WdTjP9Q39TwpNLHxcmnEM
NItQp6Pm9sqWWawIFOcx/LV4gaBUk7usadnniyz3lSrnooG0fVH9DVNwWdlnMR6I
6t0+qYpdQLu2zM339CSD11nBAnDuHizgKj7qWLMCjvHm0/rJfsE/wLSqggsQFim2
0qZIB1yzQe5OQPke5voYiCua0kFqKibvuUkTmMwggAwosER+bV22li1rxRbYhea5
fFZxx0QDxUhBtIiHuvipbDhXTwNqnYl4CfkG9/b8TmeYPKpQkU1O3rIbzCrOXbcg
bUk6OHkCgYEA4xl4WfVqFEfrvmD+KSU3KTZxRSCof8AFOfmrmJPYqocxDw9JaFPH
7fxFEP/8BnxDBcnW+ikGB8VfWNrm40LAme7TD1Ogva9ddWlE/nnOacPWX4ep00e9
ErwIYLlvOjreH5J3PBGqNDVjavO+kREKKUqXDXY7m55OY5dEPckMLhsCgYEAn1GH
CoIwXfms7+CHPDld0bS6VecJGy6MJ3kvrVyc3Toopygnafj12epKKiusExu5EOH4
JIxfm/m6XMOLixIXAa0ZWBfVrek9Bt0CWf2l3jJeyD+oRk7h9Mw8dc4FpP8acZVg
AyD/MKW2zIfjp7hi4v4vagz9JUm4i3vbyIqcFO0CgYEA1BWxQ7Hhg1c3XfAO7DYJ
Mb/aQIijU8rsFpyIGFHagkcHFd1c3MWBbUuu5JVrtFLP9NPupGkzbIZy6PRls89f
N2LGUQX0k7D2QvQwrsbqcfOmfEih3OKePKTF3i7PJT5cund6SurkXSWO1w8S5T9Y
kf9K2hOUz1wkMPXPkTP04AkCgYAnPGDRibZ3rmGUwesMPeSJHMU3Gqr3csM5hXLk
cwZ+xS/12sG6K4IApN6W/CJook81hTEjbx6svxfSeKYJHe9kjkjLlTMenW5WHl/R
4dHTovwMvQCoMA0dyJ6rNI3XUKwmhO8cVigCxwz52g2K5LIVzRvINmKxqDI2x84c
2WYPEQKBgAU26m/CfLEANjrN2gHVXaBA6uKnUnyQBl99PXH/HOJ7g8o8OXJVJIDe
taxXqti9HRcj+oSgKNlL4A4W7pUmDh5sv6tHIx16MVoJO++2CjQN8V+m9DjPlgEx
+UVmw9npr/ilfWf3MnrUdl/MefdPUDZQ9rMIaDvX3VFA9H1KsJqv
-----END RSA PRIVATE KEY-----
"""

IAP_VALID_AUDIENCE = '/projects/1234/global/backendServices/1234'
IAP_INVALID_AUDIENCE = '/projects/foo/global/backendServices/bar'

OIDC_CLIENT_ID = '123456'
OIDC_VALID_AUDIENCE = OIDC_CLIENT_ID

# Defaults for Google Cloud Identity-Aware Proxy
IAP_PUBLIC_KEY_URL = 'https://www.gstatic.com/iap/verify/public_key'
IAP_VALID_ISSUER = 'https://cloud.google.com/iap'
IAP_JWT_ALGORITHM = 'ES256'

# Defaults for Google OAuth/OpenID Connect
OIDC_PUBLIC_KEY_URL = 'https://www.googleapis.com/oauth2/v3/certs'
OIDC_VALID_ISSUER = 'https://accounts.google.com'
OIDC_JWT_ALGORITHM = 'RS256'


# pylint: disable=unused-argument
def mock_fetch_iap_public_keys(unused_argument):
    """Mock getting public keys.

    Returns:
        Mock public key.
    """
    return {"iap_1234": MOCK_EC_PUBLIC_KEY}


# pylint: disable=unused-argument
def mock_fetch_oidc_public_keys(unused_argument):
    """Mock getting public keys.

    Returns:
        Mock public key.
    """
    return {"keys": [MOCK_RSA_PUBLIC_KEY_JWK]}


def create_default_payload(audience, issuer):
    now = int(time.time())
    payload = {
        "sub": "1234567890",
        "email": "test@example.com",
        "hd": "example.com",
        "iat": now - 300,  # issued 5 min ago
        "exp": now + 300,  # expires in 5 min
        "aud": audience,
        "iss": issuer
    }
    return payload


def create_default_header(algorithm, key_id):
    header = {
        "alg": algorithm,
        "kid": key_id
    }
    return header


def create_mock_jwt(key, algorithm, key_id, audience, issuer, header=None,
                    payload=None):
    """Create test JSON Web Token (JWT).

    Args:
        key: Private key for signing.
        algorithm: Algorithm used for the private key.
        key_id: Key ID for the public key.
        header: Dict with optional header.
        payload: Dict with optional payload.

    Returns:
        Encoded JWT signed with key.
    """
    if not header:
        header = create_default_header(algorithm=algorithm, key_id=key_id)

    if not payload:
        payload = create_default_payload(audience, issuer)

    return jwt.encode(
        payload, key, algorithm=algorithm, headers=header)


@mock.patch(
    'timesketch.lib.google_auth._fetch_public_keys', mock_fetch_iap_public_keys)
class TestGoogleCloudIAP(BaseTest):
    """Tests for the functionality of the google_auth module."""

    def _test_payload_raises_jwt_validation_error(self, payload, domain=None):
        """Test JWT with supplied payload."""
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER,
            payload=payload)
        public_key = get_public_key_for_jwt(test_jwt, IAP_PUBLIC_KEY_URL)
        with self.assertRaises(JwtValidationError):
            test_decoded_jwt = decode_jwt(
                test_jwt, public_key, IAP_JWT_ALGORITHM, IAP_VALID_AUDIENCE)
            validate_jwt(test_decoded_jwt, IAP_VALID_ISSUER, domain)

    def _test_header_raises_jwt_validation_error(self, header):
        """Test JWT with supplied header."""
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER, header=header)
        public_key = get_public_key_for_jwt(test_jwt, IAP_PUBLIC_KEY_URL)

        with self.assertRaises(JwtValidationError):
            test_decoded_jwt = decode_jwt(
                test_jwt, public_key, IAP_JWT_ALGORITHM, IAP_VALID_AUDIENCE)
            validate_jwt(test_decoded_jwt, IAP_VALID_ISSUER)

    def test_valid_jwt(self):
        """Test to validate a valid JWT."""
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER)
        public_key = get_public_key_for_jwt(test_jwt, IAP_PUBLIC_KEY_URL)
        test_decoded_jwt = decode_jwt(
            test_jwt, public_key, IAP_JWT_ALGORITHM, IAP_VALID_AUDIENCE)
        validate_jwt(test_decoded_jwt, IAP_VALID_ISSUER)
        self.assertIsInstance(test_decoded_jwt, dict)
        self.assertEqual(test_decoded_jwt.get('email'), 'test@example.com')

    def test_invalid_audience_raises_jwt_validation_error(self):
        """Test to validate a JWT with wrong audience."""
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER)
        public_key = get_public_key_for_jwt(test_jwt, IAP_PUBLIC_KEY_URL)
        self.assertRaises(
            JwtValidationError, decode_jwt, test_jwt, public_key,
            IAP_JWT_ALGORITHM, IAP_INVALID_AUDIENCE)

    def test_invalid_algorithm_raises_jwt_validation_error(self):
        """Test to validate a JWT with invalid algorithm."""
        header = create_default_header(IAP_JWT_ALGORITHM, 'iap_1234')
        header['alg'] = 'HS256'
        self._test_header_raises_jwt_validation_error(header)

    def test_missing_key_id_raises_jwt_key_error(self):
        """Test to validate a JWT with key ID missing."""
        header = create_default_header(IAP_JWT_ALGORITHM, 'iap_1234')
        del header['kid']
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER, header=header)
        self.assertRaises(
            JwtKeyError, get_public_key_for_jwt, test_jwt, IAP_PUBLIC_KEY_URL)

    def test_missing_email_raises_jwt_validation_error(self):
        """Test to validate a JWT with email missing."""
        payload = create_default_payload(IAP_VALID_AUDIENCE, IAP_VALID_ISSUER)
        del payload['email']
        self._test_payload_raises_jwt_validation_error(payload)

    def test_missing_issuer_raises_jwt_validation_error(self):
        """Test to validate a JWT with issuer missing."""
        payload = create_default_payload(IAP_VALID_AUDIENCE, IAP_VALID_ISSUER)
        del payload['iss']
        self._test_payload_raises_jwt_validation_error(payload)

    def test_invalid_issuer_raises_jwt_validation_error(self):
        """Test to validate a JWT with invalid issuer"""
        payload = create_default_payload(IAP_VALID_AUDIENCE, 'invalid_issuer')
        self._test_payload_raises_jwt_validation_error(payload)

    def test_issued_in_the_future_raises_jwt_validation_error(self):
        """Test to validate a JWT created in the future."""
        payload = create_default_payload(IAP_VALID_AUDIENCE, IAP_VALID_ISSUER)
        payload['iat'] = payload['iat'] + 600  # seconds
        self._test_payload_raises_jwt_validation_error(payload)

    def test_expired_jwt_raises_jwt_validation_error(self):
        """Test to validate an expired JWT."""
        payload = create_default_payload(IAP_VALID_AUDIENCE, IAP_VALID_ISSUER)
        payload['exp'] = payload['exp'] - 600  # seconds
        self._test_payload_raises_jwt_validation_error(payload)

    def test_valid_domain(self):
        """Test to validate a JWT with domain."""
        valid_domain = 'example.com'
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER)
        public_key = get_public_key_for_jwt(test_jwt, IAP_PUBLIC_KEY_URL)
        test_decoded_jwt = decode_jwt(
            test_jwt, public_key, IAP_JWT_ALGORITHM, IAP_VALID_AUDIENCE)
        validate_jwt(test_decoded_jwt, IAP_VALID_ISSUER, valid_domain)
        self.assertIsInstance(test_decoded_jwt, dict)
        self.assertEqual(test_decoded_jwt.get('hd'), 'example.com')

    def test_invalid_domain_raises_jwt_validation_error(self):
        """Test to validate a JWT with an invalid domain."""
        invalid_domain = 'foobar.com'
        payload = create_default_payload(IAP_VALID_AUDIENCE, IAP_VALID_ISSUER)
        self._test_payload_raises_jwt_validation_error(payload, domain=invalid_domain)

    def test_invalid_public_key_raises_jwt_validation_error(self):
        """Test to validate a JWT with wrong public key."""
        test_jwt = create_mock_jwt(
            MOCK_EC_PRIVATE_KEY, algorithm=IAP_JWT_ALGORITHM, key_id='iap_1234',
            audience=IAP_VALID_AUDIENCE, issuer=IAP_VALID_ISSUER)
        self.assertRaises(
            JwtValidationError, decode_jwt, test_jwt,
            MOCK_INVALID_EC_PUBLIC_KEY, IAP_JWT_ALGORITHM, IAP_VALID_AUDIENCE)


@mock.patch(
    'timesketch.lib.google_auth._fetch_public_keys',
    mock_fetch_oidc_public_keys)
class TestGoogleCloudOpenIdConnect(BaseTest):
    """Tests for the functionality of the google_auth module."""
    def test_fetching_oidc_keys(self):
        """Test to fetch OpenID Connect formatted keys."""
        test_jwt = create_mock_jwt(
            MOCK_RSA_PRIVATE_KEY_PEM, algorithm=OIDC_JWT_ALGORITHM,
            key_id='oidc_1234', audience=OIDC_VALID_AUDIENCE,
            issuer=OIDC_VALID_ISSUER)
        public_key = get_public_key_for_jwt(test_jwt, OIDC_PUBLIC_KEY_URL)
        self.assertIsInstance(public_key, _RSAPublicKey)

    def test_valid_oidc_jwt(self):
        """Test to validate a valid OpenID Connect JWT."""
        test_jwt = create_mock_jwt(
            MOCK_RSA_PRIVATE_KEY_PEM, algorithm=OIDC_JWT_ALGORITHM,
            key_id='oidc_1234', audience=OIDC_VALID_AUDIENCE,
            issuer=OIDC_VALID_ISSUER)
        public_key = get_public_key_for_jwt(test_jwt, OIDC_PUBLIC_KEY_URL)
        test_decoded_jwt = decode_jwt(
            test_jwt, public_key, OIDC_JWT_ALGORITHM, OIDC_VALID_AUDIENCE)
        validate_jwt(test_decoded_jwt, OIDC_VALID_ISSUER)

        self.assertIsInstance(test_decoded_jwt, dict)
        self.assertEqual(test_decoded_jwt.get('email'), 'test@example.com')



# Copyright 2015 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test for the ACL model."""

from __future__ import unicode_literals

from timesketch.lib.testlib import BaseTest


class AclModelTest(BaseTest):
    """Test the ACL model."""

    def test_change_permission(self):
        """Test changing permissions on a sketch with ACL."""
        for permission in ('read', 'write', 'delete'):
            self.sketch1.grant_permission(
                permission=permission, user=self.user1)
            self.assertTrue(
                self.sketch1.has_permission(
                    permission=permission, user=self.user1))
            self.sketch1.revoke_permission(
                permission=permission, user=self.user1)
            self.assertFalse(
                self.sketch1.has_permission(
                    permission=permission, user=self.user1))

            # Test group permissions
            self.sketch1.grant_permission(
                permission=permission, group=self.group1)
            self.assertTrue(
                self.sketch1.has_permission(
                    permission=permission, user=self.user1))
            self.sketch1.revoke_permission(
                permission=permission, group=self.group1)
            self.assertFalse(
                self.sketch1.has_permission(
                    permission=permission, user=self.user1))

    def test_change_public(self):
        """Test toggle the public permission on a sketch."""
        self.sketch1.grant_permission(permission='read')
        self.assertTrue(self.sketch1.is_public)
        self.sketch1.revoke_permission(permission='read')
        self.assertFalse(self.sketch1.is_public)



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Feed aggregation.

This file provides a manual feed aggregator. This is useful for analyzers
that have already done the aggregation inside the analyzer, as part of the
analysis work they do. Instead of adding attributes to all events or
redo the aggregation each time the analyzer can simply store the aggregation
results as part of the parameters to the manual aggregator. That way the
aggregation can be used to generate charts and tables in stories and stored
insights. This analyzer is not useful for UI and therefore is "hidden" from
views.
"""

from __future__ import unicode_literals

from timesketch.lib.aggregators import manager
from timesketch.lib.aggregators import interface


class ManualFeedAggregation(interface.BaseAggregator):
    """Manual Feed Aggregation."""

    NAME = 'manual_feed'
    DISPLAY_NAME = 'Manual Feed Aggregation'
    DESCRIPTION = 'Aggregating values of a user supplied data'

    SUPPORTED_CHARTS = frozenset(
        ['barchart', 'circlechart', 'hbarchart', 'table'])

    # No Form fields since this is not meant to be used in the UI.
    FORM_FIELDS = []

    def __init__(self, sketch_id=None, index=None):
        """Initialize the aggregator object.

        Args:
            sketch_id: Sketch ID.
            index: List of elasticsearch index names.
        """
        super(ManualFeedAggregation, self).__init__(
            sketch_id=sketch_id, index=index)
        self.title = ''

    @property
    def chart_title(self):
        """Returns a title for the chart."""
        if self.title:
            return self.title
        return 'Results From A Manually Fed Table'

    # pylint: disable=arguments-differ
    def run(
            self, data, title='', supported_charts='table', field=None,
            order_field='count'):
        """Run the aggregation.

        Args:
            data: list of dict objects, each entry in the list is a
                dictionary, representing a single entry in the aggregation.
            title: string with the aggregation title.
            supported_charts: Chart type to render. Defaults to table.
            field: What field to aggregate on.
            order_field: The name of the field that is used for the order
                of items in the aggregation, defaults to "count".

        Returns:
            Instance of interface.AggregationResult with aggregation result.

        Raises:
            ValueError: data is not supplied.
        """
        if not data:
            raise ValueError('Data is missing')

        self.title = title

        if not field:
            keys = set()
            for row in data:
                list(map(keys.add, row.keys()))
            keys.discard('count')
            keys.discard('bucket_name')
            if keys:
                field = list(keys)[0]
            else:
                field = 'count'

        # Encoding information for Vega-Lite.
        encoding = {
            'x': {
                'field': field,
                'type': 'nominal',
                'sort': {
                    'op': 'sum',
                    'field': order_field,
                    'order': 'descending'
                }
            },
            'y': {'field': 'count', 'type': 'quantitative'}
        }

        return interface.AggregationResult(
            encoding=encoding, values=data, chart_type=supported_charts)


manager.AggregatorManager.register_aggregator(
    ManualFeedAggregation, exclude_from_list=True)



# Copyright 2014 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Entry point for the application."""

from __future__ import unicode_literals

import logging
import os
import sys

import six

from flask import Flask
from celery import Celery

from flask_login import LoginManager
from flask_migrate import Migrate
from flask_restful import Api
from flask_wtf import CSRFProtect

from timesketch.api.v1.routes import API_ROUTES as V1_API_ROUTES
from timesketch.lib.errors import ApiHTTPError
from timesketch.models import configure_engine
from timesketch.models import init_db
from timesketch.models.user import User
from timesketch.views.auth import auth_views
from timesketch.views.spa import spa_views


def create_app(config=None):
    """Create the Flask app instance that is used throughout the application.

    Args:
        config: Path to configuration file as a string or an object with config
        directives.

    Returns:
        Application object (instance of flask.Flask).
    """
    template_folder = 'frontend/dist'
    static_folder = 'frontend/dist'

    app = Flask(
        __name__,
        template_folder=template_folder,
        static_folder=static_folder
    )

    if not config:
        # Where to find the config file
        default_path = '/etc/timesketch/timesketch.conf'
        # Fall back to legacy location of the config file
        legacy_path = '/etc/timesketch.conf'
        if os.path.isfile(default_path):
            config = default_path
        else:
            config = legacy_path

    if isinstance(config, six.text_type):
        os.environ['TIMESKETCH_SETTINGS'] = config
        try:
            app.config.from_envvar('TIMESKETCH_SETTINGS')

            if 'EMAIL_USER_WHITELIST' in app.config:
                sys.stderr.write(
                    'Warning, EMAIL_USER_WHITELIST has been deprecated. '
                    'Please update timesketch.conf.')
        except IOError:
            sys.stderr.write(
                'Config file {0} does not exist.\n'.format(config))
            sys.exit()
    else:
        app.config.from_object(config)

    # Make sure that SECRET_KEY is configured.
    if not app.config['SECRET_KEY']:
        sys.stderr.write('ERROR: Secret key not present. '
                         'Please update your configuration.\n'
                         'To generate a key you can use openssl:\n\n'
                         '$ openssl rand -base64 32\n\n')
        sys.exit()

    # Plaso version that we support
    if app.config['UPLOAD_ENABLED']:
        try:
            # pylint: disable=import-outside-toplevel
            from plaso import __version__ as plaso_version
            app.config['PLASO_VERSION'] = plaso_version
        except ImportError:
            pass

    # Setup the database.
    configure_engine(app.config['SQLALCHEMY_DATABASE_URI'])
    db = init_db()

    # Alembic migration support:
    # http://alembic.zzzcomputing.com/en/latest/
    migrate = Migrate()
    migrate.init_app(app, db)

    # Register blueprints. Blueprints are a way to organize your Flask
    # Flask application. See this for more information:
    # http://flask.pocoo.org/docs/latest/blueprints/
    app.register_blueprint(spa_views)
    app.register_blueprint(auth_views)

    # Setup URL routes for the API.
    api_v1 = Api(app, prefix='/api/v1')
    for route in V1_API_ROUTES:
        api_v1.add_resource(*route)

    # Register error handlers
    # pylint: disable=unused-variable
    @app.errorhandler(ApiHTTPError)
    def handle_api_http_error(error):
        """Error handler for API HTTP errors.

        Returns:
            HTTP response object (instance of flask.wrappers.Response)
        """
        return error.build_response()

    # Setup the login manager.
    login_manager = LoginManager()
    login_manager.init_app(app)
    login_manager.login_view = 'user_views.login'

    # This is used by the flask_login extension.
    # pylint: disable=unused-variable
    @login_manager.user_loader
    def load_user(user_id):
        """Based on a user_id (database primary key for a user) this function
        loads a user from the database. It is used by the Flask-Login extension
        to setup up the session for the user.

        Args:
            user_id: Integer primary key for the user.

        Returns:
            A user object (Instance of timesketch.models.user.User).
        """
        return User.query.get(user_id)

    # Setup CSRF protection for the whole application
    CSRFProtect(app)

    return app


def configure_logger():
    """Configure the logger."""
    class NoESFilter(logging.Filter):
        """Custom filter to filter out ES logs"""
        def filter(self, record):
            """Filter out records."""
            return not record.name.lower() == 'elasticsearch'

    logger_formatter = logging.Formatter(
        '[%(asctime)s] %(name)s/%(levelname)s %(message)s')
    logger_filter = NoESFilter()
    logger_object = logging.getLogger('timesketch')

    for handler in logger_object.parent.handlers:
        handler.setFormatter(logger_formatter)
        handler.addFilter(logger_filter)


def create_celery_app():
    """Create a Celery app instance."""
    app = create_app()
    celery = Celery(app.import_name, broker=app.config['CELERY_BROKER_URL'])
    celery.conf.update(app.config)
    TaskBase = celery.Task

    # pylint: disable=no-init
    class ContextTask(TaskBase):
        """Add Flask context to the Celery tasks created."""
        abstract = True

        def __call__(self, *args, **kwargs):
            """Return Task within a Flask app context.

            Returns:
                A Task (instance of Celery.celery.Task)
            """
            with app.app_context():
                return TaskBase.__call__(self, *args, **kwargs)

    celery.Task = ContextTask
    return celery



# Copyright 2019 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for emojis support."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from timesketch.lib import emojis
from timesketch.lib.testlib import BaseTest


class TestEmojisLib(BaseTest):
    """Tests for the emojis support library."""

    def test_get_emoji(self):
        """Test getting emoji code."""
        skull_emoji = emojis.get_emoji('skull_crossbone')
        skull_code = '&#x2620'

        self.assertEqual(skull_emoji, skull_code)

        locomotive_emoji = emojis.get_emoji('LOCOMOTIVE')
        locomotive_code = '&#x1F682'
        self.assertEqual(locomotive_emoji, locomotive_code)

        does_not_exist = emojis.get_emoji('er_ekki_til')
        self.assertEqual(does_not_exist, '')

    def test_get_helper_from_unicode(self):
        """Test getting helper text from an emoji code."""
        skull_emoji = emojis.get_emoji('skull_crossbone')
        skull_helper = emojis.get_helper_from_unicode(skull_emoji)
        helper_text = 'Suspicious entry'

        self.assertEqual(skull_helper, helper_text)

        does_not_exist = emojis.get_helper_from_unicode('er_ekki_til')
        self.assertEqual(does_not_exist, '')

    def test_get_emojis_as_dict(self):
        """Test getting emojis as a dictionary."""
        skull_code = '&#x2620'
        helper_text = 'Suspicious entry'
        emojis_dict = emojis.get_emojis_as_dict()
        skull_helper = emojis_dict.get(skull_code)
        self.assertEqual(skull_helper, helper_text)

        does_not_exist = emojis_dict.get('finns_inte')
        self.assertIsNone(does_not_exist)



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module holds version 1 of the Timesketch API.

The timesketch API is a RESTful API that exposes the following resources:
"""
from __future__ import unicode_literals

import logging

from flask import current_app
from flask import jsonify
from flask_restful import fields
from flask_restful import marshal

from timesketch.lib.definitions import HTTP_STATUS_CODE_OK
from timesketch.lib.datastores.elastic import ElasticsearchDataStore


logging.basicConfig(
    format='%(asctime)s %(levelname)-8s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S')


class ResourceMixin(object):
    """Mixin for API resources."""
    # Schemas for database model resources
    user_fields = {'username': fields.String}
    group_fields = {'name': fields.String}

    aggregation_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'description': fields.String,
        'agg_type': fields.String,
        'parameters': fields.String,
        'chart_type': fields.String,
        'label_string': fields.String,
        'user': fields.Nested(user_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    aggregation_group_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'description': fields.String,
        'aggregations': fields.Nested(aggregation_fields),
        'parameters': fields.String,
        'orientation': fields.String,
        'user': fields.Nested(user_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    status_fields = {
        'id': fields.Integer,
        'status': fields.String,
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    searchindex_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'user': fields.Nested(user_fields),
        'description': fields.String,
        'index_name': fields.String,
        'status': fields.Nested(status_fields),
        'label_string': fields.String,
        'deleted': fields.Boolean,
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    timeline_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'user': fields.Nested(user_fields),
        'description': fields.String,
        'status': fields.Nested(status_fields),
        'color': fields.String,
        'label_string': fields.String,
        'searchindex': fields.Nested(searchindex_fields),
        'deleted': fields.Boolean,
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    analysis_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'description': fields.String,
        'analyzer_name': fields.String,
        'parameters': fields.String,
        'result': fields.String,
        'analysissession_id': fields.Integer,
        'log': fields.String,
        'user': fields.Nested(user_fields),
        'timeline': fields.Nested(timeline_fields),
        'status': fields.Nested(status_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    analysis_session_fields = {
        'id': fields.Integer,
        'user': fields.Nested(user_fields),
        'analyses': fields.Nested(analysis_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    searchtemplate_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'user': fields.Nested(user_fields),
        'query_string': fields.String,
        'query_filter': fields.String,
        'query_dsl': fields.String,
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    view_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'description': fields.String,
        'user': fields.Nested(user_fields),
        'query_string': fields.String,
        'query_filter': fields.String,
        'query_dsl': fields.String,
        'searchtemplate': fields.Nested(searchtemplate_fields),
        'aggregation': fields.Nested(aggregation_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    story_fields = {
        'id': fields.Integer,
        'title': fields.String,
        'content': fields.String,
        'user': fields.Nested(user_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    sketch_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'description': fields.String,
        'user': fields.Nested(user_fields),
        'timelines': fields.List(fields.Nested(timeline_fields)),
        'stories': fields.List(fields.Nested(story_fields)),
        'aggregations': fields.Nested(aggregation_fields),
        'aggregationgroups': fields.Nested(aggregation_group_fields),
        'active_timelines': fields.List(fields.Nested(timeline_fields)),
        'label_string': fields.String,
        'status': fields.Nested(status_fields),
        'all_permissions': fields.String,
        'my_permissions': fields.String,
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    comment_fields = {
        'comment': fields.String,
        'user': fields.Nested(user_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    label_fields = {
        'name': fields.String,
        'user': fields.Nested(user_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    graph_fields = {
        'id': fields.Integer,
        'name': fields.String,
        'user': fields.Nested(user_fields),
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    graphcache_fields = {
        'id': fields.Integer,
        'graph_elements': fields.String,
        'created_at': fields.DateTime,
        'updated_at': fields.DateTime
    }

    fields_registry = {
        'aggregation': aggregation_fields,
        'aggregationgroup': aggregation_group_fields,
        'searchindex': searchindex_fields,
        'analysis': analysis_fields,
        'analysissession': analysis_session_fields,
        'timeline': timeline_fields,
        'searchtemplate': searchtemplate_fields,
        'view': view_fields,
        'user': user_fields,
        'graph': graph_fields,
        'graphcache': graphcache_fields,
        'group': group_fields,
        'sketch': sketch_fields,
        'story': story_fields,
        'event_comment': comment_fields,
        'event_label': label_fields
    }

    @property
    def datastore(self):
        """Property to get an instance of the datastore backend.

        Returns:
            Instance of timesketch.lib.datastores.elastic.ElasticSearchDatastore
        """
        return ElasticsearchDataStore(
            host=current_app.config['ELASTIC_HOST'],
            port=current_app.config['ELASTIC_PORT'])

    def to_json(self,
                model,
                model_fields=None,
                meta=None,
                status_code=HTTP_STATUS_CODE_OK):
        """Create json response from a database models.

        Args:
            model: Instance of a timesketch database model
            model_fields: Dictionary describing the resulting schema
            meta: Dictionary holding any metadata for the result
            status_code: Integer used as status_code in the response

        Returns:
            Response in json format (instance of flask.wrappers.Response)
        """
        if not meta:
            meta = dict()

        schema = {'meta': meta, 'objects': []}

        if model:
            if not model_fields:
                try:
                    model_fields = self.fields_registry[model.__tablename__]
                except AttributeError:
                    model_fields = self.fields_registry[model[0].__tablename__]
            schema['objects'] = [marshal(model, model_fields)]

        response = jsonify(schema)
        response.status_code = status_code
        return response



# Copyright 2015 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for the auth views."""

from __future__ import unicode_literals

from flask import current_app
from flask_login import current_user

from timesketch.lib.definitions import HTTP_STATUS_CODE_REDIRECT
from timesketch.lib.testlib import BaseTest


class AuthViewTest(BaseTest):
    """Test the auth view."""

    def test_login_view_unauthenticated(self):
        """Test the login view handler with an unauthenticated session."""
        response = self.client.get('/login/')
        self.assert200(response)

    def test_login_view_form_authenticated(self):
        """Test the login view handler with an authenticated session."""
        self.login()
        response = self.client.get('/login/')
        self.assertEqual(response.status_code, HTTP_STATUS_CODE_REDIRECT)

    def test_login_view_sso_authenticated(self):
        """Test the login view handler with an SSO authenticated session."""
        current_app.config['SSO_ENABLED'] = True
        current_app.config['SSO_GROUP_ENV_VARIABLE'] = 'SSO_GROUP'
        current_app.config['SSO_GROUP_SEPARATOR'] = ';'
        current_app.config['SSO_GROUP_NOT_MEMBER_SIGN'] = '-'
        with self.client:
            response = self.client.get(
                '/login/',
                environ_base={
                    'REMOTE_USER': 'test1',
                    'SSO_GROUP': 'test_group1;-test_group2'
                })
            self.assertEqual(current_user.username, 'test1')
            self.assertIn(self.group1, current_user.groups)
            self.assertNotIn(self.group2, current_user.groups)
            self.assertEqual(response.status_code, HTTP_STATUS_CODE_REDIRECT)

    def test_logout_view(self):
        """Test the logout view handler."""
        self.login()
        response = self.client.get('/logout/')
        self.assertEqual(response.status_code, HTTP_STATUS_CODE_REDIRECT)



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Graph plugin for Windows logins."""

from timesketch.lib.graphs.interface import BaseGraphPlugin
from timesketch.lib.graphs import manager


class WinLoginsGraph(BaseGraphPlugin):
    """Graph plugin for Windows logins."""

    NAME = 'WinLogins'
    DISPLAY_NAME = 'Windows logins'

    def generate(self):
        """Generate the graph.

        Returns:
            Graph object instance.
        """
        query = 'tag:logon-event'
        return_fields = [
            'computer_name', 'username', 'logon_type', 'logon_process'
        ]

        events = self.event_stream(
            query_string=query, return_fields=return_fields)

        for event in events:
            computer_name = event['_source'].get('computer_name')
            username = event['_source'].get('username')
            logon_type = event['_source'].get('logon_type')

            computer = self.graph.add_node(computer_name, {'type': 'computer'})
            user = self.graph.add_node(username, {'type': 'user'})
            self.graph.add_edge(user, computer, logon_type, event)

        self.graph.commit()

        return self.graph


manager.GraphManager.register_graph(WinLoginsGraph)



# Copyright 2019 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for chart manager."""

from __future__ import unicode_literals

from timesketch.lib.testlib import BaseTest
from timesketch.lib.charts import manager


class MockChart(object):
    """Mock chart class."""
    NAME = 'MockChart'


class TestChartManager(BaseTest):
    """Tests for the functionality of the manager module."""

    manager.ChartManager.clear_registration()
    manager.ChartManager.register_chart(MockChart)

    def test_get_charts(self):
        """Test to get chart class objects."""
        charts = manager.ChartManager.get_charts()
        chart_list = list(charts)
        first_chart_tuple = chart_list[0]
        chart_name, chart_class = first_chart_tuple
        self.assertIsInstance(chart_list, list)
        self.assertIsInstance(first_chart_tuple, tuple)
        self.assertEqual(chart_class, MockChart)
        self.assertEqual(chart_name, 'mockchart')

    def test_get_chart(self):
        """Test to get chart class from registry."""
        chart_class = manager.ChartManager.get_chart('mockchart')
        self.assertEqual(chart_class, MockChart)
        self.assertRaises(
            KeyError, manager.ChartManager.get_chart, 'no_such_chart')

    def test_register_chart(self):
        """Test so we raise KeyError when chart is already registered."""
        self.assertRaises(
            KeyError, manager.ChartManager.register_chart, MockChart)



"""Sketch analyzer plugin for feature extraction."""
from __future__ import unicode_literals

import logging
import re

import six

from timesketch.lib import emojis
from timesketch.lib.analyzers import interface
from timesketch.lib.analyzers import manager


logger = logging.getLogger('timesketch.analyzers.feature')
RE_FLAGS = [
    're.ASCII',
    're.IGNORECASE',
    're.LOCALE',
    're.MULTILINE',
    're.DOTALL',
    're.VERBOSE',
]


class FeatureExtractionSketchPlugin(interface.BaseSketchAnalyzer):
    """Sketch analyzer for FeatureExtraction."""

    NAME = 'feature_extraction'

    CONFIG_FILE = 'features.yaml'

    FORM_FIELDS = [
        {
            'name': 'query_string',
            'type': 'ts-dynamic-form-text-input',
            'label': 'The filter query to narrow down the result set',
            'placeholder': 'Query',
            'default_value': ''
        },
        {
            'name': 'query_dsl',
            'type': 'ts-dynamic-form-text-input',
            'label': 'The filter query DSL to narrow down the result',
            'placeholder': 'Query DSL',
            'default_value': ''
        },
        {
            'name': 'attribute',
            'type': 'ts-dynamic-form-text-input',
            'label': 'Name of the field to apply regular expression against',
            'placeholder': 'Field Name',
            'default_value': ''
        },
        {
            'name': 'store_as',
            'type': 'ts-dynamic-form-text-input',
            'label': 'Name of the field to store the extracted results in',
            'placeholder': 'Store results as field name',
            'default_value': ''
        },
        {
            'name': 're',
            'type': 'ts-dynamic-form-text-input',
            'label': 'The regular expression to extract data from field',
            'placeholder': 'Regular Expression',
            'default_value': ''
        },
        {
            'name': 're_flags',
            'type': 'ts-dynamic-form-multi-select-input',
            'label': 'List of flags to pass to the regular expression',
            'placeholder': 'Regular Expression flags',
            'default_value': [],
            'options': RE_FLAGS,
            'optional': True,
        },
        {
            'name': 'emojis',
            'type': 'ts-dynamic-form-multi-select-input',
            'label': 'List of emojis to add to events with matches',
            'placeholder': 'Emojis to add to events',
            'default_value': [],
            'options': [x.code for x in emojis.EMOJI_MAP.values()],
            'options-label': [
                '{0:s} - {1:s}'.format(
                    x, y.help) for x, y in emojis.EMOJI_MAP.items()],
            'optional': True,
        },
        {
            'name': 'tags',
            'type': 'ts-dynamic-form-text-input',
            'label': 'Tag to add to events with matches',
            'placeholder': 'Tag to add to events',
            'default_value': '',
            'optional': True,
        },
        {
            'name': 'create_view',
            'type': 'ts-dynamic-form-boolean',
            'label': 'Should a view be created if there is a match',
            'placeholder': 'Create a view',
            'default_value': False,
            'optional': True,
        },
        {
            'name': 'store_type_list',
            'type': 'ts-dynamic-form-boolean',
            'label': 'Store extracted result in type List',
            'placeholder': 'Store results as field type list',
            'default_value': False,
            'optional': True,
        },
        {
            'name': 'overwrite_store_as',
            'type': 'ts-dynamic-form-boolean',
            'label': 'Overwrite the field to store if already exist',
            'placeholder': 'Overwrite the field to store',
            'default_value': True,
            'optional': True,
        },
        {
            'name': 'overwrite_and_merge_store_as',
            'type': 'ts-dynamic-form-boolean',
            'label': 'Overwrite the field to store and merge value if exist',
            'placeholder': 'Overwrite the field to store and merge value',
            'default_value': False,
            'optional': True,
        },
        {
            'name': 'keep_multimatch',
            'type': 'ts-dynamic-form-boolean',
            'label': 'Keep multi match datas',
            'placeholder': 'Keep multi match',
            'default_value': False,
            'optional': True,
        },
        {
            'name': 'aggregate',
            'type': 'ts-dynamic-form-boolean',
            'label': 'Should results be aggregated if there is a match',
            'placeholder': 'Aggregate results',
            'default_value': False,
            'optional': True,
        },
    ]


    def __init__(self, index_name, sketch_id, config=None):
        """Initialize The Sketch Analyzer.

        Args:
            index_name: Elasticsearch index name
            sketch_id: Sketch ID
            config: Optional dict that contains the configuration for the
                analyzer. If not provided, the default YAML file will be
                loaded up.
        """
        self.index_name = index_name
        super().__init__(
            index_name, sketch_id)
        self._config = config

    def run(self):
        """Entry point for the analyzer.

        Returns:
            String with summary of the analyzer result.
        """
        config = self._config or interface.get_yaml_config(self.CONFIG_FILE)
        if not config:
            return 'Unable to parse the config file.'

        return_strings = []
        for name, feature_config in iter(config.items()):
            feature_string = self.extract_feature(name, feature_config)
            if feature_string:
                return_strings.append(feature_string)

        return ', '.join(return_strings)

    @staticmethod
    def _get_attribute_value(
            current_val,
            extracted_value,
            keep_multi,
            merge_values,
            type_list):
        """Returns the attribute value as it should be stored.

        Args:
            current_val: current value of store_as.
            extracted_value: values matched from regexp (type list).
            keep_multi: choise if you keep all match from regex (type boolean).
            merge_values: choise if you merge value from extracted
                 and current (type boolean).
            type_list: choise if you store values in list type(type boolean).

        Returns:
            Value to store
        """
        if not current_val:
            merge_values = False
        if len(extracted_value) == 1:
            keep_multi = False
        if type_list:
            if merge_values and keep_multi:
                return sorted(list(set(current_val) | set(extracted_value)))
            if merge_values:
                if extracted_value[0] not in current_val:
                    current_val.append(extracted_value[0])
                return sorted(current_val)
            if keep_multi:
                return sorted(extracted_value)
            return [extracted_value[0]]
        if merge_values and keep_multi:
            list_cur = current_val.split(',')
            merge_list = sorted(list(set(list_cur) | set(extracted_value)))
            return ','.join(merge_list)
        if merge_values:
            if extracted_value[0] in current_val:
                return current_val
            return f'{current_val},{extracted_value[0]}'
        if keep_multi:
            return ','.join(extracted_value)
        return extracted_value[0]

    def extract_feature(self, name, config):
        """Extract features from events.

        Args:
            name: String with the name describing the feature to be extracted.
            config: A dict that contains the configuration for the feature
                extraction. See data/features.yaml for fields and further
                documentation of what needs to be defined.

        Returns:
            String with summary of the analyzer result.
        """
        query = config.get('query_string')
        query_dsl = config.get('query_dsl')
        attribute = config.get('attribute')
        store_type_list = config.get('store_type_list', False)
        keep_multimatch = config.get('keep_multimatch', False)
        overwrite_store_as = config.get('overwrite_store_as', True)
        overwrite_and_merge_store_as = config.get(
            'overwrite_and_merge_store_as', False)

        if not attribute:
            logger.warning('No attribute defined.')
            return ''

        store_as = config.get('store_as')
        if not store_as:
            logger.warning('No attribute defined to store results in.')
            return ''

        tags = config.get('tags', [])

        expression_string = config.get('re')
        expression_flags = config.get('re_flags')
        if not expression_string:
            logger.warning('No regular expression defined.')
            return ''

        if expression_flags:
            flags = set()
            for flag in expression_flags:
                try:
                    flags.add(getattr(re, flag))
                except AttributeError:
                    logger.warning('Unknown regular expression flag defined.')
                    return ''
            re_flag = sum(flags)
        else:
            re_flag = 0

        try:
            expression = re.compile(expression_string, flags=re_flag)
        except re.error as exception:
            # pylint: disable=logging-format-interpolation
            logger.warning((
                'Regular expression failed to compile, with '
                'error: {0!s}').format(exception))
            return ''

        emoji_names = config.get('emojis', [])
        emojis_to_add = [emojis.get_emoji(x) for x in emoji_names]

        return_fields = [attribute, store_as]

        events = self.event_stream(
            query_string=query, query_dsl=query_dsl,
            return_fields=return_fields)

        event_counter = 0
        for event in events:
            attribute_field = event.source.get(attribute)
            if isinstance(attribute_field, six.text_type):
                attribute_value = attribute_field
            elif isinstance(attribute_field, (list, tuple)):
                attribute_value = ','.join(attribute_field)
            elif isinstance(attribute_field, (int, float)):
                attribute_value = attribute_field
            else:
                attribute_value = None

            if not attribute_value:
                continue

            result = expression.findall(attribute_value)
            if not result:
                continue
            result = list(set(result))

            event_counter += 1
            store_as_current_val = event.source.get(store_as)
            if store_as_current_val and not overwrite_store_as:
                continue
            if isinstance(store_as_current_val, six.text_type):
                store_type_list = False
            elif isinstance(store_as_current_val, (list, tuple)):
                store_type_list = True
            new_value = self._get_attribute_value(
                store_as_current_val,
                result,
                keep_multimatch,
                overwrite_and_merge_store_as,
                store_type_list
                )
            if not new_value:
                continue
            event.add_attributes({store_as: new_value})
            event.add_emojis(emojis_to_add)
            event.add_tags(tags)

            # Commit the event to the datastore.
            event.commit()

        aggregate_results = config.get('aggregate', False)
        create_view = config.get('create_view', False)

        # If aggregation is turned on, we automatically create an aggregation.
        if aggregate_results:
            create_view = True

        if create_view and event_counter:
            view = self.sketch.add_view(
                name, self.NAME, query_string=query, query_dsl=query_dsl)

            if aggregate_results:
                params = {
                    'field': store_as,
                    'limit': 20,
                }
                self.sketch.add_aggregation(
                    name='Top 20 for: {0:s} [{1:s}]'.format(store_as, name),
                    agg_name='field_bucket', agg_params=params,
                    description='Created by the feature extraction analyzer',
                    view_id=view.id, chart_type='hbarchart')

        return 'Feature extraction [{0:s}] extracted {1:d} features.'.format(
            name, event_counter)


manager.AnalysisManager.register_analyzer(FeatureExtractionSketchPlugin)



# Copyright 2015 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module implements HTTP request handlers for the user views."""

from __future__ import unicode_literals

import requests

from flask import abort
from flask import Blueprint
from flask import current_app
from flask import redirect
from flask import render_template
from flask import request
from flask import session
from flask import url_for

from oauthlib import oauth2

from flask_login import current_user
from flask_login import login_user
from flask_login import logout_user

from timesketch.lib.definitions import HTTP_STATUS_CODE_UNAUTHORIZED
from timesketch.lib.definitions import HTTP_STATUS_CODE_BAD_REQUEST
from timesketch.lib.definitions import HTTP_STATUS_CODE_OK
from timesketch.lib.forms import UsernamePasswordForm
from timesketch.lib.google_auth import get_public_key_for_jwt
from timesketch.lib.google_auth import get_oauth2_discovery_document
from timesketch.lib.google_auth import get_oauth2_authorize_url
from timesketch.lib.google_auth import get_encoded_jwt_over_https
from timesketch.lib.google_auth import decode_jwt
from timesketch.lib.google_auth import validate_jwt
from timesketch.lib.google_auth import JwtValidationError
from timesketch.lib.google_auth import JwtKeyError
from timesketch.lib.google_auth import JwtFetchError
from timesketch.lib.google_auth import DiscoveryDocumentError
from timesketch.lib.google_auth import CSRF_KEY
from timesketch.models import db_session
from timesketch.models.user import Group
from timesketch.models.user import User

# Register flask blueprint
auth_views = Blueprint('user_views', __name__)

TOKEN_URI = 'https://www.googleapis.com/oauth2/v3/tokeninfo'
SCOPES = [
    'https://www.googleapis.com/auth/userinfo.email',
    'openid',
    'https://www.googleapis.com/auth/userinfo.profile']


@auth_views.route('/login/', methods=['GET', 'POST'])
def login():
    """Handler for the login page view.

    There are three ways of authentication.
    1) Google Cloud Identity-Aware Proxy.
    2) If Single Sign On (SSO) is enabled in the configuration and the
       environment variable is present, e.g. REMOTE_USER then the system will
       get or create the user object and setup a session for the user.
    3) Local authentication is used if SSO login is not enabled. This will
       authenticate the user against the local user database.

    Returns:
        Redirect if authentication is successful or template with context
        otherwise.
    """
    # Google OpenID Connect authentication.
    if current_app.config.get('GOOGLE_OIDC_ENABLED', False):
        hosted_domain = current_app.config.get('GOOGLE_OIDC_HOSTED_DOMAIN')
        return redirect(get_oauth2_authorize_url(hosted_domain))

    # Google Identity-Aware Proxy authentication (using JSON Web Tokens)
    if current_app.config.get('GOOGLE_IAP_ENABLED', False):
        encoded_jwt = request.environ.get(
            'HTTP_X_GOOG_IAP_JWT_ASSERTION', None)
        if encoded_jwt:
            expected_audience = current_app.config.get('GOOGLE_IAP_AUDIENCE')
            expected_issuer = current_app.config.get('GOOGLE_IAP_ISSUER')
            algorithm = current_app.config.get('GOOGLE_IAP_ALGORITHM')
            url = current_app.config.get('GOOGLE_IAP_PUBLIC_KEY_URL')
            try:
                public_key = get_public_key_for_jwt(encoded_jwt, url)
                decoded_jwt = decode_jwt(
                    encoded_jwt, public_key, algorithm, expected_audience)
                validate_jwt(decoded_jwt, expected_issuer)
                email = decoded_jwt.get('email')
                if email:
                    user = User.get_or_create(username=email, name=email)
                    login_user(user)

            except (ImportError, NameError, UnboundLocalError):
                raise

            except (JwtValidationError, JwtKeyError, Exception) as e:  # pylint: disable=broad-except
                current_app.logger.error('{}'.format(e))

    # SSO login based on environment variable, e.g. REMOTE_USER.
    if current_app.config.get('SSO_ENABLED', False):
        remote_user_env = current_app.config.get('SSO_USER_ENV_VARIABLE',
                                                 'REMOTE_USER')
        sso_group_env = current_app.config.get('SSO_GROUP_ENV_VARIABLE', None)

        remote_user = request.environ.get(remote_user_env, None)
        if remote_user:
            user = User.get_or_create(username=remote_user, name=remote_user)
            login_user(user)

        # If we get groups from the SSO system create the group(s) in
        # Timesketch and add/remove the user from it.
        if sso_group_env:
            groups_string = request.environ.get(sso_group_env, '')
            separator = current_app.config.get('SSO_GROUP_SEPARATOR', ';')
            not_member_sign = current_app.config.get(
                'SSO_GROUP_NOT_MEMBER_SIGN', None)
            for group_name in groups_string.split(separator):
                remove_group = False
                if not_member_sign:
                    remove_group = group_name.startswith(not_member_sign)
                    group_name = group_name.lstrip(not_member_sign)

                # Get or create the group in the Timesketch database.
                group = Group.get_or_create(name=group_name)

                if remove_group:
                    if group in user.groups:
                        user.groups.remove(group)
                else:
                    if group not in user.groups:
                        user.groups.append(group)
            # Commit the changes to the database.
            db_session.commit()

    # Login form POST
    form = UsernamePasswordForm()
    if form.validate_on_submit:
        user = User.query.filter_by(username=form.username.data).first()
        if user:
            if user.check_password(plaintext=form.password.data):
                login_user(user)

    # Log the user in and setup the session.
    if current_user.is_authenticated:
        return redirect(request.args.get('next') or '/')

    return render_template('login.html', form=form)


@auth_views.route('/logout/', methods=['GET'])
def logout():
    """Handler for the logout page view.

    Returns:
        Redirect response.
    """
    logout_user()
    return redirect(url_for('user_views.login'))


@auth_views.route('/login/api_callback/', methods=['GET'])
def validate_api_token():
    """Handler for logging in using an authenticated session for the API.

    Returns:
        A simple page indicating the user is authenticated.
    """
    try:
        token = oauth2.rfc6749.tokens.get_token_from_header(request)
    except AttributeError:
        token = None

    if not token:
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED, 'Request not authenticated.')

    id_token = request.args.get('id_token')
    if not id_token:
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED, 'No ID token supplied.')

    client_id = current_app.config.get('GOOGLE_OIDC_API_CLIENT_ID')
    if not client_id:
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST,
            'No OIDC API client ID defined in the configuration file.')

    # Authenticating session, see more details here:
    # https://www.oauth.com/oauth2-servers/signing-in-with-google/\
    #     verifying-the-user-info/
    # Sending a request to Google to verify that the access token
    # is valid, to be able to validate the session.
    data = {
        'access_token': token}
    bearer_token_response = requests.post(TOKEN_URI, data=data)
    if bearer_token_response.status_code != HTTP_STATUS_CODE_OK:
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST, 'Unable to validate access token.')
    bearer_token_json = bearer_token_response.json()

    data = {
        'id_token': id_token
    }
    token_response = requests.post(TOKEN_URI, data=data)
    token_json = token_response.json()

    verified = token_json.get('email_verified', False)
    if not verified:
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Session not authenticated or account not verified')

    if bearer_token_json.get('azp', 'a') != token_json.get('azp', 'x'):
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Auth token and client tokens don\'t match, azp differs.')

    if bearer_token_json.get('email', 'a') != token_json.get('email', 'b'):
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Auth token and client tokens don\'t match, email differs.')

    try:
        discovery_document = get_oauth2_discovery_document()
    except DiscoveryDocumentError as e:
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST,
            'Unable to discover document, with error: {0!s}'.format(e))

    expected_issuer = discovery_document['issuer']
    try:
        validate_jwt(token_json, expected_issuer)
    except (ImportError, NameError, UnboundLocalError):
        raise
    except (JwtValidationError, JwtKeyError, Exception) as e:  # pylint: disable=broad-except
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Unable to validate the JWT token, with error: {0!s}.'.format(e))

    read_client_id = token_json.get('aud', '')
    if read_client_id != client_id:
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Client ID {0:s} does not match server configuration for '
            'client'.format(read_client_id))

    read_scopes = bearer_token_json.get('scope', '').split()
    if not set(read_scopes) == set(SCOPES):
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Client scopes differ from what they should be (email, openid, '
            'profile) = {} VS {}'.format(SCOPES, read_scopes))

    validated_email = token_json.get('email')

    # Check if the authenticating user is part of the allowed domains.
    hosted_domains = current_app.config.get('GOOGLE_OIDC_HOSTED_DOMAIN')
    if hosted_domains:
        _, _, domain = validated_email.partition('@')
        if domain.lower() != hosted_domains.lower():
            return abort(
                HTTP_STATUS_CODE_UNAUTHORIZED,
                'Domain {0:s} is not allowed to authenticate against this '
                'instance.'.format(domain))

    allowed_users = current_app.config.get('GOOGLE_OIDC_ALLOWED_USERS')
    # TODO: Remove that after a 6 months, this following check is to ensure
    # compatibility of config file
    if not allowed_users:
        current_app.logger.warning('Warning, GOOGLE_OIDC_USER_WHITELIST has '
                                   'been deprecated. Please update '
                                   'timesketch.conf.')
        allowed_users = current_app.config.get('GOOGLE_OIDC_USER_WHITELIST', [])

    # Check if the authenticating user is on the allow list.
    if allowed_users:
        if validated_email not in allowed_users:
            return abort(
                HTTP_STATUS_CODE_UNAUTHORIZED,
                'Unauthorized request, user not allowed')

    user = User.get_or_create(username=validated_email, name=validated_email)
    login_user(user)

    # Log the user in and setup the session.
    if current_user.is_authenticated:
        return """
<h1>Authenticated</h1>
        """

    return abort(
        HTTP_STATUS_CODE_BAD_REQUEST, 'User is not authenticated.')


@auth_views.route('/login/google_openid_connect/', methods=['GET'])
def google_openid_connect():
    """Handler for the Google OpenID Connect callback.

    Reference:
    https://developers.google.com/identity/protocols/OpenIDConnect

    Returns:
        Redirect response.
    """
    error = request.args.get('error', None)

    if error:
        current_app.logger.error('OAuth2 flow error: {}'.format(error))
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST,
            'OAuth2 flow error: {0!s}'.format(error))

    try:
        code = request.args['code']
        client_csrf_token = request.args.get('state')
        server_csrf_token = session[CSRF_KEY]
    except KeyError as e:
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST,
            'Client CSRF error, no CSRF key stored')

    if client_csrf_token != server_csrf_token:
        return abort(HTTP_STATUS_CODE_BAD_REQUEST, 'Invalid CSRF token')

    try:
        encoded_jwt = get_encoded_jwt_over_https(code)
    except JwtFetchError as e:
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST,
            'Jwt Fetch error, {0!s}'.format(e))

    try:
        discovery_document = get_oauth2_discovery_document()
    except DiscoveryDocumentError as e:
        return abort(
            HTTP_STATUS_CODE_BAD_REQUEST,
            'Unable to discover document, with error: {0!s}'.format(e))

    algorithm = discovery_document['id_token_signing_alg_values_supported'][0]
    expected_audience = current_app.config.get('GOOGLE_OIDC_CLIENT_ID')
    expected_domain = current_app.config.get('GOOGLE_OIDC_HOSTED_DOMAIN')
    expected_issuer = discovery_document['issuer']

    # Fetch the public key and try to validate the JWT.
    try:
        public_key = get_public_key_for_jwt(
            encoded_jwt, discovery_document['jwks_uri'])
        decoded_jwt = decode_jwt(
            encoded_jwt, public_key, algorithm, expected_audience)
        validate_jwt(
            decoded_jwt, expected_issuer, expected_domain)
    except (JwtValidationError, JwtKeyError) as e:
        current_app.logger.error('{}'.format(e))
        return abort(
            HTTP_STATUS_CODE_UNAUTHORIZED,
            'Unable to validate request, with error: {0!s}'.format(e))

    validated_email = decoded_jwt.get('email')
    allowed_users = current_app.config.get('GOOGLE_OIDC_ALLOWED_USERS')

    # Check if the authenticating user is allowed.
    if allowed_users:
        if validated_email not in allowed_users:
            return abort(
                HTTP_STATUS_CODE_UNAUTHORIZED,
                'Unauthorized request, user not allowed')

    user = User.get_or_create(username=validated_email, name=validated_email)
    login_user(user)

    # Log the user in and setup the session.
    if current_user.is_authenticated:
        return redirect(request.args.get('next') or '/')

    return abort(
        HTTP_STATUS_CODE_BAD_REQUEST, 'User is not authenticated.')



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for graph manager."""

from timesketch.lib.testlib import BaseTest
from timesketch.lib.graphs import manager


class MockGraph:
    """Mock graph class."""
    NAME = 'MockGraph'
    DISPLAY_NAME = 'MockGraph'


class TestGraphManager(BaseTest):
    """Tests for the functionality of the manager module."""

    manager.GraphManager.clear_registration()
    manager.GraphManager.register_graph(MockGraph)

    def test_get_graphs(self):
        """Test to get graph class objects."""
        graphs = manager.GraphManager.get_graphs()
        graph_list = list(graphs)
        first_graph_tuple = graph_list[0]
        graph_name, graph_class = first_graph_tuple
        self.assertIsInstance(graph_list, list)
        self.assertIsInstance(first_graph_tuple, tuple)
        self.assertEqual(graph_class, MockGraph)
        self.assertEqual(graph_name, 'mockgraph')

    def test_get_graph(self):
        """Test to get graph class from registry."""
        graph_class = manager.GraphManager.get_graph('mockgraph')
        self.assertEqual(graph_class, MockGraph)
        self.assertRaises(
            KeyError, manager.GraphManager.get_graph, 'no_such_graph')

    def test_register_graph(self):
        """Test so we raise KeyError when graph is already registered."""
        self.assertRaises(
            KeyError, manager.GraphManager.register_graph, MockGraph)



"""Sketch analyzer plugin for domains."""
from __future__ import unicode_literals

import collections
import difflib

import logging

from flask import current_app
from datasketch.minhash import MinHash

from timesketch.lib import emojis
from timesketch.lib import similarity
from timesketch.lib.analyzers import interface
from timesketch.lib.analyzers import manager
from timesketch.lib.analyzers import utils


logger = logging.getLogger('timesketch.analyzers.phishy_domains')


class PhishyDomainsSketchPlugin(interface.BaseSketchAnalyzer):
    """Sketch analyzer for phishy domains."""

    NAME = 'phishy_domains'

    DEPENDENCIES = frozenset(['domain'])

    # This list contains entries from Alexa top 10 list (as of 2018-12-27).
    # They are used to create the base of a domain watch list. For custom
    # entries use DOMAIN_ANALYZER_WATCHED_DOMAINS in timesketch.conf.
    WATCHED_DOMAINS_BASE_LIST = [
        'google.com', 'youtube.com', 'facebook.com', 'baidu.com',
        'wikipedia.org', 'qq.com', 'amazon.com', 'yahoo.com', 'taobao.com',
        'reddit.com']

    def __init__(self, index_name, sketch_id):
        """Initialize The Sketch Analyzer.

        Args:
            index_name: Elasticsearch index name
            sketch_id: Sketch ID
        """
        self.index_name = index_name
        super(PhishyDomainsSketchPlugin, self).__init__(index_name, sketch_id)

        self.domain_scoring_threshold = current_app.config.get(
            'DOMAIN_ANALYZER_WATCHED_DOMAINS_SCORE_THRESHOLD', 0.75)
        self.domain_scoring_exclude_domains = current_app.config.get(
            'DOMAIN_ANALYZER_EXCLUDE_DOMAINS', [])

        # TODO: remove that after a 6 months, this following check is to ensure
        # compatibility of the config file.
        if len(self.domain_scoring_exclude_domains) == 0:
            logger.warning(
                'Warning, DOMAIN_ANALYZER_WHITELISTED_DOMAINS has been '
                'deprecated. Please update timesketch.conf.')
            self.domain_scoring_exclude_domains = current_app.config.get(
                'DOMAIN_ANALYZER_WHITELISTED_DOMAINS', [])


    @staticmethod
    def _get_minhash_from_domain(domain):
        """Get the Minhash value from a domain name.

        This function takes a domain, removes the TLD extension
        from it and then creates a MinHash object from every
        remaining character in the domain.

        If a domain starts with www., it will be stripped of the
        domain before the Minhash is calculated.

        Args:
          domain: string with a full domain, eg. www.google.com

        Returns:
            A minhash (instance of datasketch.minhash.MinHash)
        """
        domain_items = domain.split('.')
        domain_part = '.'.join(domain_items[:-1])

        minhash = MinHash(similarity.DEFAULT_PERMUTATIONS)
        for char in domain_part:
            minhash.update(char.encode('utf8'))

        return minhash

    def _get_similar_domains(self, domain, domain_dict):
        """Compare a domain to a list of domains and return similar domains.

        This function takes a domain and a dict object that contains
        as key domain names and value the calculated MinHash value for that
        domain as well as the domains depth (mbl.is is 2, foobar.evil.com would
        be 3). It will then strip www. if needed from the domain, and compare
        the Jaccard distance between all domains in the dict and the supplied
        domain (removing the TLD extension from all domains).

        If the Jaccard distance between the supplied domain and one or more of
        the domains in the domain dict is higher than the configured threshold
        the domain is further tested to see if there are overlapping substrings
        between the two domains. If there is a common substring that is longer
        than half the domain name and the Jaccard distance is above the
        threshold the domain is considered to be similar.

        Args:
            domain: string with a full domain, eg. www.google.com
            domain_dict: dict with domain names (keys) and MinHash objects
                (values) for all domains to compare against.

        Returns:
            a list of tuples (score, similar_domain_name) with the names of
            the similar domains as well as the Jaccard distance between
            the supplied domain and the matching one.
        """
        domain = utils.strip_www_from_domain(domain)

        similar = []
        if '.' not in domain:
            return similar

        if domain in domain_dict:
            return similar

        if any(domain.endswith('.{0:s}'.format(x)) for x in domain_dict):
            return similar

        # We want to get rid of the TLD extension of the domain.
        # This is only used in the substring match in case the Jaccard
        # distance is above the threshold.
        domain_items = domain.split('.')
        domain_depth = len(domain_items)
        domain_part = '.'.join(domain_items[:-1])

        minhashes = {}
        for index in range(0, domain_depth - 1):
            minhashes[domain_depth - index] = self._get_minhash_from_domain(
                '.'.join(domain_items[index:]))

        for watched_domain, watched_item in iter(domain_dict.items()):
            watched_hash = watched_item.get('hash')
            watched_depth = watched_item.get('depth')

            minhash = minhashes.get(watched_depth)
            if not minhash:
                # The supplied domains length does not match this watched
                # domain.
                continue
            score = watched_hash.jaccard(minhash)
            if score < self.domain_scoring_threshold:
                continue

            watched_domain_items = watched_domain.split('.')
            watched_domain_part = '.'.join(watched_domain_items[:-1])

            # Check if there are also any overlapping strings.
            sequence = difflib.SequenceMatcher(
                None, domain_part, watched_domain_part)
            match = sequence.find_longest_match(
                0, len(domain_part), 0, len(watched_domain_part))

            # We want to have at least half of the domain matching.
            # TODO: This can be improved, this is a value and part that
            # needs or can be tweaked. Perhaps move this to a config option
            # that is the min length of strings.
            match_size = min(
                int(len(domain_part)/2), int(len(watched_domain_part)/2))
            if match.size < match_size:
                continue
            similar.append((watched_domain, score))

        return similar

    def run(self):
        """Entry point for the analyzer.

        Returns:
            String with summary of the analyzer result
        """
        query = (
            '{"query": { "bool": { "should": [ '
            '{ "exists" : { "field" : "url" }}, '
            '{ "exists" : { "field" : "domain" }} ] } } }')

        return_fields = ['domain', 'url', 'message', 'human_readable']

        events = self.event_stream(
            '', query_dsl=query, return_fields=return_fields)

        domains = {}
        domain_counter = collections.Counter()
        tld_counter = collections.Counter()

        for event in events:
            domain = event.source.get('domain')

            if not domain:
                continue

            domain_counter[domain] += 1
            domains.setdefault(domain, [])
            domains[domain].append(event)

            tld = utils.get_tld_from_domain(domain)
            tld_counter[tld] += 1

        if not domain_counter:
            return 'No domains discovered, so no phishy domains.'

        watched_domains_list = current_app.config.get(
            'DOMAIN_ANALYZER_WATCHED_DOMAINS', [])
        domain_threshold = current_app.config.get(
            'DOMAIN_ANALYZER_WATCHED_DOMAINS_THRESHOLD', 10)
        watched_domains_list.extend([
            utils.strip_www_from_domain(x)
            for x, _ in domain_counter.most_common(domain_threshold)])
        watched_domains_list.extend([
            x for x, _ in tld_counter.most_common(domain_threshold)])
        watched_domains_list.extend(self.WATCHED_DOMAINS_BASE_LIST)
        watched_domains_list_temp = set(watched_domains_list)
        watched_domains_list = []
        for domain in watched_domains_list_temp:
            if domain in self.domain_scoring_exclude_domains:
                continue
            if any(domain.endswith(x)
                   for x in self.domain_scoring_exclude_domains):
                continue

            if '.' not in domain:
                continue
            watched_domains_list.append(domain)

        watched_domains = {}
        for domain in watched_domains_list:
            minhash = self._get_minhash_from_domain(domain)
            watched_domains[domain] = {
                'hash': minhash,
                'depth': len(domain.split('.'))
            }

        similar_domain_counter = 0
        allowlist_encountered = False
        evil_emoji = emojis.get_emoji('SKULL_CROSSBONE')
        phishing_emoji = emojis.get_emoji('FISHING_POLE')
        for domain, _ in iter(domain_counter.items()):
            emojis_to_add = []
            tags_to_add = []
            text = None

            similar_domains = self._get_similar_domains(
                domain, watched_domains)

            if similar_domains:
                similar_domain_counter += 1
                emojis_to_add.append(evil_emoji)
                emojis_to_add.append(phishing_emoji)
                tags_to_add.append('phishy-domain')
                similar_text_list = ['{0:s} [score: {1:.2f}]'.format(
                    phishy_domain,
                    score) for phishy_domain, score in similar_domains]
                text = 'Domain {0:s} is similar to {1:s}'.format(
                    domain, ', '.join(similar_text_list))
                if any(domain.endswith(
                        x) for x in self.domain_scoring_exclude_domains):
                    tags_to_add.append('known-domain')
                    allowlist_encountered = True

            for event in domains.get(domain, []):
                event.add_emojis(emojis_to_add)
                event.add_tags(tags_to_add)
                if text:
                    event.add_human_readable(text, self.NAME, append=False)

                # Commit the event to the datastore.
                event.commit()

        if similar_domain_counter:
            self.sketch.add_view(
                view_name='Phishy Domains', analyzer_name=self.NAME,
                query_string='tag:"phishy-domain"')

            if allowlist_encountered:
                self.sketch.add_view(
                    view_name='Phishy Domains, excl. known domains',
                    analyzer_name=self.NAME,
                    query_string=(
                        'tag:"phishy-domain" AND NOT tag:"known-domain"'))

        return (
            '{0:d} potentially phishy domains discovered.').format(
                similar_domain_counter)


manager.AnalysisManager.register_analyzer(PhishyDomainsSketchPlugin)



# Copyright 2019 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This file contains a class for managing charts."""

from __future__ import unicode_literals


class ChartManager(object):
    """The chart manager."""

    _class_registry = {}

    @classmethod
    def get_charts(cls):
        """Retrieves the registered charts.

        Yields:
            tuple: containing:
                unicode: the uniquely identifying name of the chart
                type: the chart class.
        """
        for agg_name, agg_class in iter(cls._class_registry.items()):
            yield agg_name, agg_class

    @classmethod
    def get_chart(cls, chart_name):
        """Retrieves a class object of a specific chart.

        Args:
            chart_name (unicode): name of the chart to retrieve.

        Returns:
            Chart class object or None if not existing.
        """
        try:
            chart_class = cls._class_registry[chart_name.lower()]
        except KeyError:
            raise KeyError(
                'No such chart type: {0:s}'.format(chart_name.lower()))
        return chart_class

    @classmethod
    def register_chart(cls, chart_class):
        """Registers a chart class.

        The chart classes are identified by their lower case name.

        Args:
            chart_class (type): the chart class to register.

        Raises:
            KeyError: if class is already set for the corresponding name.
        """
        chart_name = chart_class.NAME.lower()
        if chart_name in cls._class_registry:
            raise KeyError('Class already set for name: {0:s}.'.format(
                chart_class.NAME))
        cls._class_registry[chart_name] = chart_class

    @classmethod
    def clear_registration(cls):
        """Clears all chart registrations."""
        cls._class_registry = {}



# Copyright 2020 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for story export manager."""

from __future__ import unicode_literals

from timesketch.lib.testlib import BaseTest
from timesketch.lib.stories import manager


class MockStoryExporter(object):
    """Mock story exporter class,"""
    EXPORT_FORMAT = 'MockFormat'


class MockStoryExporter2(object):
    """Mock story exporter class,"""
    EXPORT_FORMAT = 'MockFormat2'


class MockStoryExporter3(object):
    """Mock story exporter class,"""
    EXPORT_FORMAT = 'MockFormat3'


class TestStoryExportManager(BaseTest):
    """Tests for the functionality of the manager module."""

    def setUp(self):
        """Set up the tests."""
        super(TestStoryExportManager, self).setUp()
        manager.StoryExportManager.clear_registration()
        manager.StoryExportManager.register_exporter(MockStoryExporter)

    def test_get_exporters(self):
        """Test to get exporter class objects."""
        exporters = manager.StoryExportManager.get_exporters()
        exporter_list = list(exporters)
        self.assertIsInstance(exporter_list, list)
        exporter_dict = {}
        for name, exporter_class in exporter_list:
            exporter_dict[name] = exporter_class
        self.assertIn('mockformat', exporter_dict)
        exporter_class = exporter_dict.get('mockformat')
        self.assertEqual(exporter_class, MockStoryExporter)

        manager.StoryExportManager.clear_registration()
        exporters = manager.StoryExportManager.get_exporters()
        exporter_list = [x for x, _ in exporters]
        self.assertEqual(exporter_list, [])

        manager.StoryExportManager.clear_registration()
        manager.StoryExportManager.register_exporter(MockStoryExporter)
        manager.StoryExportManager.register_exporter(MockStoryExporter2)
        manager.StoryExportManager.register_exporter(MockStoryExporter3)

        exporters = manager.StoryExportManager.get_exporters()
        exporter_list = [x for x, _ in exporters]
        self.assertEqual(len(exporter_list), 3)
        self.assertIn('mockformat', exporter_list)

    def test_get_exporter(self):
        """Test to get exporter class from registry."""
        exporter_class = manager.StoryExportManager.get_exporter('mockformat')
        self.assertEqual(exporter_class, MockStoryExporter)

    def test_register_exporter(self):
        """Test so we raise KeyError when exporter is already registered."""
        self.assertRaises(
            KeyError, manager.StoryExportManager.register_exporter,
            MockStoryExporter)



"""Sketch analyzer plugin for login and logoff events."""
from __future__ import unicode_literals

import logging

import six

from timesketch.lib import emojis
from timesketch.lib.analyzers import interface
from timesketch.lib.analyzers import manager


logger = logging.getLogger('timesketch.analyzers.login')
LOGON_TYPES = {
    '0': 'Unknown',
    '2': 'Interactive',
    '3': 'Network',
    '4': 'Batch',
    '5': 'Service',
    '7': 'Unlock',
    '8': 'NetworkCleartext',
    '9': 'NewCredentials',
    '10': 'RemoteInteractive',
    '11': 'CachedInteractive'
}


def parse_evtx_logoff_event(string_list):
    """Parse logoff events and return a dict with attributes.

    Args:
        string_list: a list of strings extracted from the Event Log.

    Returns:
        Dict with attributes parsed out of the logoff events.
    """
    if not len(string_list) == 5:
        return {}

    attributes = {}
    attributes['username'] = string_list[1]
    attributes['logon_domain'] = string_list[2]
    attributes['session_id'] = string_list[3]

    logon_type_code = string_list[4]
    attributes['logon_type'] = LOGON_TYPES.get(
        logon_type_code, LOGON_TYPES.get('0'))

    return attributes


def parse_evtx_logon_event(string_list, string_parsed):
    """Parse logon events and return a count of event processed.

    Args:
        string_list: a list of strings extracted from the Event Log.
        string_parsed: a dict with strings extracted from the Event log.

    Returns:
        Dict with attributes parsed out of the logon events.
    """
    if len(string_list) < 20:
        return {}

    if not string_parsed:
        string_parsed = {}
        string_parsed['target_user_id'] = string_list[4]
        string_parsed['target_user_name'] = string_list[5]
        string_parsed['hostname'] = string_list[11]
        string_parsed['source_user_name'] = string_list[1]

    attributes = {}
    logon_type_code = string_list[8]
    attributes['logon_type'] = LOGON_TYPES.get(
        logon_type_code, LOGON_TYPES.get('0'))

    win_domain = string_list[2]
    if win_domain:
        attributes['windows_domain'] = win_domain

    username = string_parsed.get('target_user_name')
    if username:
        attributes['username'] = username

    user_id = string_parsed.get('target_user_id')
    if user_id:
        attributes['user_id'] = user_id

    logon_process_name = string_list[9]
    if logon_process_name:
        attributes['logon_process'] = logon_process_name

    workstation_name = string_list[11]
    if workstation_name == '-':
        attributes['workstation'] = 'localhost'
    elif workstation_name:
        attributes['workstation'] = workstation_name

    ip_address = string_list[18]
    if ip_address and ip_address != '-':
        attributes['source_address'] = ip_address

    hostname = string_parsed.get('target_machine_name', 'N/A')
    if hostname:
        attributes['hostname'] = hostname

    session_id = string_list[3]
    if session_id:
        attributes['session_id'] = session_id

    source_username = string_parsed.get('source_user_name')
    if source_username:
        attributes['source_username'] = source_username

    return attributes


class LoginSketchPlugin(interface.BaseSketchAnalyzer):
    """Sketch analyzer for Login and Logoff related activity."""

    NAME = 'login'
    DEPENDENCIES = frozenset()

    def __init__(self, index_name, sketch_id):
        """Initialize The Sketch Analyzer.

        Args:
            index_name: Elasticsearch index name
            sketch_id: Sketch ID
        """
        self.index_name = index_name
        super(LoginSketchPlugin, self).__init__(index_name, sketch_id)

    def run(self):
        """Entry point for the analyzer.

        Returns:
            String with summary of the analyzer result
        """
        login_emoji = emojis.get_emoji('unlock')
        logoff_emoji = emojis.get_emoji('lock')
        screen_emoji = emojis.get_emoji('screen')
        screensaver_logon = LOGON_TYPES.get('7')
        login_counter = 0
        logoff_counter = 0

        # TODO: Add EVT lookups, ID 528 for logon and 538, 540 for logoff.
        # TODO: Add RDP EVT lookups, ID 682 for logon and 683 for logoff.
        query = (
            'data_type:"windows:evtx:record" AND (event_identifier:4624 OR '
            'event_identifier:4778 OR event_identifier:4779 OR '
            'event_identifier:4634 OR event_identifier:4647)')

        return_fields = [
            'message', 'data_type', 'strings', 'strings_parsed',
            'event_identifier']

        # Generator of events based on your query.
        events = self.event_stream(
            query_string=query, return_fields=return_fields)

        for event in events:
            strings = event.source.get('strings')
            strings_parsed = event.source.get('strings_parsed')
            identifier = event.source.get('event_identifier')
            emojis_to_add = []
            tags_to_add = []
            attribute_dict = {}

            if isinstance(identifier, six.text_type):
                try:
                    identifier = int(identifier, 10)
                except ValueError:
                    logger.warning((
                        'Unable to convert EVTX identifier to an integer, '
                        'value is {0:s}').format(identifier))
                    continue

            if identifier == 4624:
                attribute_dict = parse_evtx_logon_event(
                    strings, strings_parsed)
                if not attribute_dict:
                    continue
                emojis_to_add.append(login_emoji)
                tags_to_add.append('logon-event')
                login_counter += 1

            elif identifier in (4634, 4647):
                attribute_dict = parse_evtx_logoff_event(strings)
                if not attribute_dict:
                    continue
                emojis_to_add.append(logoff_emoji)
                tags_to_add.append('logoff-event')
                logoff_counter += 1

            # TODO: Add support for RDP events, ID 4778 (logon) and 4779
            # (logoff).
            if not attribute_dict:
                continue
            event.add_attributes(attribute_dict)

            # Want to add an emoji in case this is a screensaver unlock.
            if attribute_dict.get('logon_type', '') == screensaver_logon:
                emojis_to_add.append(screen_emoji)

            event.add_emojis(emojis_to_add)
            event.add_tags(tags_to_add)

            # Commit the event to the datastore.
            event.commit()

        # TODO: Add support for Linux syslog logon/logoff events.
        # TODO: Add support for Mac OS X logon/logoff events.

        return (
            'Total number of login events processed: {0:d} and '
            'logoff events: {1:d}').format(login_counter, logoff_counter)


manager.AnalysisManager.register_analyzer(LoginSketchPlugin)
